@@ -55,6 +55,7 @@ enum
 	SECURITY_CHANGE,
 	ZOOM_CHANGE,
 	FAVICON,
+	POPUP_BLOCKED,
 	LAST_SIGNAL
 };
 
@@ -260,6 +261,16 @@ galeon_embed_base_init (gpointer g_class)
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_STRING);
+	galeon_embed_signals[POPUP_BLOCKED] =
+                g_signal_new ("ge_popupblocked",
+                              GALEON_EMBED_TYPE,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonEmbedClass, popupblocked),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1,
+			      G_TYPE_STRING);
 	initialized = TRUE;
 	}
 }
@@ -550,10 +561,13 @@ galeon_embed_shistory_go_nth (GaleonEmbed *embed,
 
 gboolean
 galeon_embed_shistory_copy (GaleonEmbed *source,
-                            GaleonEmbed *dest)
+                            GaleonEmbed *dest,
+			    gboolean back_history,
+			    gboolean forward_history,
+			    gboolean set_current)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (source);
-        return klass->shistory_copy (source, dest);
+        return klass->shistory_copy (source, dest, back_history, forward_history, set_current);
 }
 
 gresult
