@@ -210,6 +210,9 @@ struct GaleonEmbedClass
 	void (* favicon)	 (GaleonEmbed *embed,
 			          const char *uri);
 
+	void (* popupblocked)	 (GaleonEmbed *embed,
+				  const char *uri);
+
 	/* Methods  */
         void (* get_capabilities)          (GaleonEmbed *embed,
 				            EmbedCapabilities *caps);
