@@ -154,12 +154,15 @@ extern "C" {
 #include "galeon-embed-shell.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
+#include "gul-download.h"
 #include "gul-glade.h"
 #include "gul-string.h"
 #include "gul-general.h"
+#include "gul-gui.h"
 #include "galeon-embed-utils.h"
 #include "ProgressListener.h"
 #include "ContentHandler.h"
+#include "disclosure-widget.h"
 
 #include <gtk/gtkentry.h>
 #include <gtk/gtktogglebutton.h>
@@ -176,11 +179,9 @@ extern "C" {
 
 #include "nsCRT.h"
 #include "nsCOMPtr.h"
-#include "nsIFactory.h"
 #include "nsISupportsArray.h"
 #include "nsIServiceManager.h"
 #include "nsWeakReference.h"
-#include "nsXPComFactory.h"
 
 #include "nsString.h"
 #include "nsIURI.h"
@@ -214,13 +215,31 @@ gint mime_ask_dialog_cancel_clicked_cb (GtkButton *button,
  */
 struct MimeAskActionDialog
 {
-	gchar *url;
-	gchar *mime_type;
+	MimeAskActionDialog(GContentHandler *aContentHandler,
+			    GtkWidget *aParentWidget,
+			    const char *aMimeType);
+	~MimeAskActionDialog();
+
 	GContentHandler *mContentHandler;
-	GladeXML *gxml;
-	GtkWidget *mRememberCheck;
-	GtkWidget *parent;
-	char *scheme;
+	GladeXML *mGXml;
+	GtkWidget *mDialogWidget;
+	GtkWidget *mParent;
+	GtkWidget *mAppMenu;
+	GtkWidget *mToggleButton;
+	GtkWidget *mListView;
+	GtkWidget *mScrolledView;
+
+	GList *mApps;
+	GtkListStore *mListStore;
+
+	enum
+	{
+		APP_COLUMN,
+		N_COLUMNS
+	};
+
+	// How many rows to show in the applications list
+	static const int NUM_APPLICATIONS = 4; 
 };
 
 /* Implementation file */
@@ -272,7 +291,7 @@ NS_IMETHODIMP GContentHandler::
 
 	mContext = aWindowContext;
 
-	nsCOMPtr<nsIDOMWindowInternal> aWindowInternal = 
+	nsCOMPtr<nsIDOMWindowInternal> windowInternal = 
 					do_QueryInterface (aWindowContext);
 	
 	nsCOMPtr<nsILocalFile> saveDir;
@@ -312,49 +331,66 @@ NS_IMETHODIMP GContentHandler::
 	saveDir->InitWithPath (NS_ConvertUTF8toUCS2(expanded_directory));
 	g_free (expanded_directory);
 
-	nsCOMPtr<nsIFilePicker> aFilePicker =
-				do_CreateInstance (G_FILEPICKER_CONTRACTID);
+	nsCOMPtr <nsILocalFile> saveFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
 
-	const nsAString &title = NS_ConvertUTF8toUCS2(_("Select the destination filename"));
-
-	aFilePicker->Init (aWindowInternal,
-			   PromiseFlatString(title).get(), 
-			   nsIFilePicker::modeSave);
-	aFilePicker->SetDefaultString (aDefaultFile);
-	aFilePicker->SetDisplayDirectory (saveDir);
-
-	nsCOMPtr <nsILocalFile> aFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
-
-	PRInt16 okToSave;
-	if (eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR))
-	{
-		aFilePicker->Show (&okToSave);
-
-		if (okToSave == nsIFilePicker::returnOK)
-		{
-			aFilePicker->GetFile (getter_AddRefs(aFile));
-		};
-	}
-	else
+	// If we aren't asking for downloading dir, check that we aren't
+	// overwriting anything.  If we are, pop up a warning dialog and show
+	// the filepicker if the user doesn't want to overwrite the file.
+	PRInt16 okToSave = nsIFilePicker::returnCancel;
+	if (!eel_gconf_get_boolean (CONF_DOWNLOADING_ASK_DIR))
 	{
 		nsAutoString uDirName;
 		saveDir->GetPath (uDirName);
-		aFile->InitWithPath (uDirName + NS_LITERAL_STRING("/") +
-				     nsDependentString(aDefaultFile));
-		okToSave = nsIFilePicker::returnOK;
-	};
 
+		const nsACString &cDirName = NS_ConvertUCS2toUTF8(uDirName);
+		const nsACString &cDefaultFile = NS_ConvertUCS2toUTF8(aDefaultFile);
+		char *filename = 
+			g_build_filename (PromiseFlatCString(cDirName).get(),
+					  PromiseFlatCString(cDefaultFile).get(),
+					  NULL);
+
+		// FIXME determine the parent widget if possible
+		if (gul_gui_confirm_overwrite_file (NULL, filename))
+		{
+			saveFile->InitWithPath (uDirName + NS_LITERAL_STRING("/") +
+					        nsDependentString(aDefaultFile));
+			okToSave = nsIFilePicker::returnOK;
+		}
+
+		g_free (filename);
+	}
+
+	if (okToSave == nsIFilePicker::returnCancel)
+	{
+		nsCOMPtr<nsIFilePicker> filePicker =
+					do_CreateInstance (G_FILEPICKER_CONTRACTID);
+
+		const nsAString &title = NS_ConvertUTF8toUCS2(_("Select the destination filename"));
+
+		filePicker->Init (windowInternal,
+				   PromiseFlatString(title).get(), 
+				   nsIFilePicker::modeSave);
+		filePicker->SetDefaultString (aDefaultFile);
+		filePicker->SetDisplayDirectory (saveDir);
+
+		filePicker->Show (&okToSave);
+
+		if (okToSave == nsIFilePicker::returnOK)
+		{
+			filePicker->GetFile (getter_AddRefs(saveFile));
+		}
+	}
 
 	if (okToSave == nsIFilePicker::returnCancel)
 		return NS_ERROR_FAILURE;
 	else
 	{
-		nsCOMPtr<nsIFile> aDirectory;
-		rv = aFile->GetParent (getter_AddRefs(aDirectory));
-		if(NS_SUCCEEDED(rv) && aDirectory)
+		nsCOMPtr<nsIFile> directory;
+		rv = saveFile->GetParent (getter_AddRefs(directory));
+		if(NS_SUCCEEDED(rv) && directory)
 		{
 			nsAutoString uDirectoryName;
-			rv = aDirectory->GetPath(uDirectoryName);
+			rv = directory->GetPath(uDirectoryName);
 			const nsACString &cDirectoryName = NS_ConvertUCS2toUTF8(uDirectoryName);
 			if(NS_SUCCEEDED(rv))
 			{
@@ -363,25 +399,17 @@ NS_IMETHODIMP GContentHandler::
 			}
 		}
 
-		NS_IF_ADDREF (*_retval = aFile);
+		NS_IF_ADDREF (*_retval = saveFile);
 		return NS_OK;
-	};
+	}
 }
 
 /* void showProgressDialog (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_METHOD GContentHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
 					      nsISupports *aContext)
 {
-#if MOZILLA_SNAPSHOT > 1
 	g_print ("GContentHandler::ShowProgressDialog is depreciated!\n");
 	return NS_ERROR_NOT_IMPLEMENTED;
-#else
-	DownloadAction aAction =
-		mHelperProgress ? ACTION_SAVEFORHELPER : ACTION_NONE;
-	GProgressListener *aListener = new GProgressListener ();
-	aListener->InitForDownload (aLauncher, aContext, this, aAction);
-	return NS_OK;
-#endif
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -390,9 +418,7 @@ NS_METHOD GContentHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
 
 NS_METHOD GContentHandler::LaunchExternalDownloader (void)
 {
-	/* FIXME external download helpers are not compiled because
-	 * they use galeon-shell code. Utils should really not do it
-	gul_download_external_save_url (mUrl.get());*/
+	gul_download_external_save_url (mUrl.get());
 	return NS_OK;
 }
 
@@ -404,7 +430,6 @@ NS_METHOD GContentHandler::FindHelperApp (void)
 		return LaunchHelperApp ();
 	}
 	else
-#if MOZILLA_SNAPSHOT > 1
 	{
 		if (NS_SUCCEEDED(SynchroniseMIMEInfo()))
 		{
@@ -412,11 +437,8 @@ NS_METHOD GContentHandler::FindHelperApp (void)
 		}
 		else
 		{
-				return NS_ERROR_FAILURE;
+			return NS_ERROR_FAILURE;
 		}
-#else
-		return ShowHelperProgressDialog ();
-#endif			
 	}
 }
 
@@ -476,21 +498,13 @@ NS_METHOD GContentHandler::GetContext (nsISupports * *_retval)
 	return NS_OK;
 }
 
-/*
-NS_METHOD GContentHandler::GetMimeItemReference (MimeItem * *_retval)
-{
-	*_retval = mMimeItem;
-	return NS_OK;
-}
-*/
-
 static gboolean 
-application_support_scheme (GnomeVFSMimeApplication *app, const char *scheme)
+application_support_scheme (GnomeVFSMimeApplication *app, const nsCString &aScheme)
 {
 	GList *l;
 
 	g_return_val_if_fail (app != NULL, FALSE);
-	g_return_val_if_fail (scheme != NULL, FALSE);
+	g_return_val_if_fail (!aScheme.IsEmpty(), FALSE);
 	
 	if (app->expects_uris != GNOME_VFS_MIME_APPLICATION_ARGUMENT_TYPE_URIS)
 		return FALSE;
@@ -499,7 +513,7 @@ application_support_scheme (GnomeVFSMimeApplication *app, const char *scheme)
 	{
 		char *uri_scheme = (char *)l->data;
 		g_return_val_if_fail (uri_scheme != NULL, FALSE);
-		if (strcmp (uri_scheme, scheme) == 0) return TRUE;
+		if (aScheme.Equals(uri_scheme)) return TRUE;
 	}
 
 	return FALSE;
@@ -508,19 +522,8 @@ application_support_scheme (GnomeVFSMimeApplication *app, const char *scheme)
 NS_METHOD GContentHandler::SetHelperApp(GnomeVFSMimeApplication *aHelperApp,
 					PRBool alwaysUse)
 {
-	char *scheme = g_strdup (mScheme.get());
-
 	mHelperApp = aHelperApp;
-	mUrlHelper = application_support_scheme (aHelperApp, scheme);
-
-	g_free (scheme);
-
-	if (nsCRT::strcmp(aHelperApp->id,"OtHeR") ==0)
-	{
-		g_free (aHelperApp->command);
-		g_free (aHelperApp->id);
-		delete aHelperApp;
-	}
+	mUrlHelper = application_support_scheme (aHelperApp, mScheme);
 
 	return NS_OK;
 }
@@ -542,7 +545,7 @@ NS_METHOD GContentHandler::SynchroniseMIMEInfo (void)
 	if(NS_FAILED(rv)) return NS_ERROR_FAILURE;	
 
 	nsMIMEInfoHandleAction mimeInfoAction;
-	mimeInfoAction = nsIMIMEInfo::alwaysAsk;
+	mimeInfoAction = nsIMIMEInfo::useHelperApp;
 
 	if(mHelperApp->requires_terminal) //Information passing kludge!
 	{
@@ -564,9 +567,9 @@ NS_METHOD GContentHandler::Init (void)
 {
 	nsresult rv;
 
-	nsCOMPtr<nsIMIMEInfo> aMIMEInfo;
-	rv = mLauncher->GetMIMEInfo (getter_AddRefs(aMIMEInfo));
-	rv = aMIMEInfo->GetMIMEType (&mMimeType);
+	nsCOMPtr<nsIMIMEInfo> MIMEInfo;
+	rv = mLauncher->GetMIMEInfo (getter_AddRefs(MIMEInfo));
+	rv = MIMEInfo->GetMIMEType (&mMimeType);
 	
 	rv = mLauncher->GetDownloadInfo(getter_AddRefs(mUri),
 					&mTimeDownloadStarted,
@@ -576,9 +579,9 @@ NS_METHOD GContentHandler::Init (void)
 #if 0
 	/* GetSource seems redundant and isn't in 0.9 This code is here while
 	   it remains unclear what GetSource is for. --phil */
-	nsCOMPtr<nsIURI> aUri;
-	rv = mLauncher->GetSource(getter_AddRefs(aUri));
-	rv = aUri->GetSpec (mUrl);
+	nsCOMPtr<nsIURI> uri;
+	rv = mLauncher->GetSource(getter_AddRefs(uri));
+	rv = uri->GetSpec (mUrl);
 #endif	
 	ProcessMimeInfo ();
 
@@ -591,11 +594,11 @@ NS_METHOD GContentHandler::ProcessMimeInfo (void)
 	    !nsCRT::strcmp(mMimeType, "application/octet-stream"))
 	{
 		nsresult rv;
-		nsCOMPtr<nsIURL> aURL = do_QueryInterface(mUri, &rv);
-		if (NS_SUCCEEDED(rv) && aURL)
+		nsCOMPtr<nsIURL> url = do_QueryInterface(mUri, &rv);
+		if (NS_SUCCEEDED(rv) && url)
 		{
 			nsCAutoString uriFileName;
-			aURL->GetFileName(uriFileName);
+			url->GetFileName(uriFileName);
 			mMimeType = g_strdup
 					(gnome_vfs_mime_type_from_name
 						(uriFileName.get()));
@@ -609,75 +612,130 @@ NS_METHOD GContentHandler::ProcessMimeInfo (void)
 
 NS_METHOD GContentHandler::MIMEAskAction (void)
 {
-	MimeAskActionDialog *dialog = new MimeAskActionDialog;
-	GtkWidget *label;
-	GtkWidget *dialogWidget;
-	const char *description;
-	char ltext[255];
-		
-	dialog->url = !mUrl.IsEmpty() ? g_strdup (mUrl.get()) : NULL;
-	dialog->mime_type = g_strdup (mMimeType);
-	dialog->mContentHandler = this;
-	dialog->gxml = gul_glade_widget_new ("galeon.glade", "mime_ask_action_dialog", 
-					     &dialogWidget, dialog);
-	dialog->mRememberCheck =
-		glade_xml_get_widget (dialog->gxml, "mime_remember_check");
-	dialog->scheme = g_strdup (mScheme.get());
+	nsCOMPtr<nsIDOMWindow> parent = do_QueryInterface(mContext);
+	GtkWidget *parentWidget = MozillaFindGtkParent(parent);
 
-#ifdef ENABLE_JUST_GNOMEVFS
-	{
-		GtkWidget *w;
-		w = glade_xml_get_widget (dialog->gxml, "mimedb_options_vbox");
-		gtk_widget_hide (w);
-	}
-#endif	
-	
-	GtkWidget *aMimeIcon = glade_xml_get_widget (dialog->gxml,
-						     "mime_ask_action_icon");
-	gtk_image_set_from_file(GTK_IMAGE(aMimeIcon),
-				gnome_vfs_mime_get_icon(mMimeType));						     
-
-	nsCOMPtr<nsIDOMWindow> parent = do_QueryInterface (mContext);
-	
-	GtkWidget *aParentWidget = MozillaFindGtkParent (parent);
-	dialog->parent = aParentWidget;
-	gtk_window_set_transient_for (GTK_WINDOW (dialogWidget), 
-				      GTK_WINDOW (aParentWidget));
-
-	description = gnome_vfs_mime_get_description (mMimeType);
-	if (!description) description = mMimeType;
-	
-	g_snprintf (ltext, 255, "<b>%s</b>", description);
-	label = glade_xml_get_widget (dialog->gxml, "mime_ask_action_description");
-	gtk_label_set_markup (GTK_LABEL (label), ltext);
-
-	gtk_widget_show(dialogWidget);
+	new MimeAskActionDialog(this, parentWidget, mMimeType);
 
 	return NS_OK;
 }
 
-//------------------------------------------------------------------------------
+////////////////////////////////////////////////////////////////////////////////
+// begin MIMEAskActionDialog methods.
+////////////////////////////////////////////////////////////////////////////////
 
-NS_DEF_FACTORY (GContentHandler, GContentHandler);
-
-/**
- * NS_NewContentHandlerFactory:
- */ 
-nsresult NS_NewContentHandlerFactory(nsIFactory** aFactory)
+MimeAskActionDialog::MimeAskActionDialog(GContentHandler *aContentHandler,
+					 GtkWidget *aParentWidget,
+					 const char *aMimeType) :
+					 mContentHandler(aContentHandler),
+					 mParent(aParentWidget)
 {
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
+	GtkWidget *label;
+	const char *description;
+	char ltext[255]; //philipl: Fixed length buffer == potential security problem...
 
-	nsGContentHandlerFactory *result = new nsGContentHandlerFactory;
-	if (result == NULL)
+	mGXml = gul_glade_widget_new("galeon.glade", "mime_ask_action_dialog", 
+				     &mDialogWidget, this);
+
+	mAppMenu = glade_xml_get_widget(mGXml, "mime_ask_dialog_app_menu");
+	mToggleButton = glade_xml_get_widget(mGXml, "mime_ask_dialog_toggle_helpers");
+	mListView = glade_xml_get_widget(mGXml, "mime_ask_dialog_helper_list_view");
+	mScrolledView = glade_xml_get_widget(mGXml, "mime_ask_dialog_scrolled_view");
+
+	cddb_disclosure_set_labels (CDDB_DISCLOSURE (mToggleButton),
+			            _("Helper Applications"),
+			            _("Helper Applications"));
+
+	mApps = gnome_vfs_mime_get_all_applications(aMimeType);
+	if (mApps)
 	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
+		GnomeVFSMimeApplication *defaultApp =
+			gnome_vfs_mime_get_default_application(aMimeType);
 
-	return NS_OK;
+		mListStore = gtk_list_store_new(N_COLUMNS,
+						G_TYPE_STRING);
+		gtk_tree_view_set_model(GTK_TREE_VIEW(mListView),
+					GTK_TREE_MODEL(mListStore));
+
+		GtkCellRenderer *renderer;
+		renderer = gtk_cell_renderer_text_new();
+
+		GtkTreeViewColumn *column;
+		column = gtk_tree_view_column_new_with_attributes("Application",
+						renderer,
+						"text", APP_COLUMN,
+						NULL);
+		gtk_tree_view_append_column(GTK_TREE_VIEW(mListView), column);
+
+		GtkTreeSelection *selection =
+				gtk_tree_view_get_selection(GTK_TREE_VIEW(mListView));
+		gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
+
+		guint index;
+		for (GList *i = mApps ; i ; i = i->next, index++)
+		{
+			GnomeVFSMimeApplication *app =
+				static_cast<GnomeVFSMimeApplication *>(i->data);
+
+			GtkTreeIter iter;
+			gtk_list_store_append (mListStore, &iter);
+			gtk_list_store_set (mListStore, &iter,
+					    APP_COLUMN, app->name,
+					    -1);
+
+			if (defaultApp &&
+			    !g_ascii_strcasecmp(app->name, defaultApp->name))
+			{
+				gtk_tree_selection_select_iter(selection, &iter);
+			}
+		}
+
+		// Calculate the correct width for the list view for showing
+		// NUM_APPLICATION rows.
+		gint width, height, ypad;
+		gtk_cell_renderer_get_size (renderer, GTK_WIDGET (mListView),
+				            NULL, NULL, NULL, &width, &height);
+		g_object_get (renderer, "ypad", &ypad, NULL);
+		height = NUM_APPLICATIONS*(height + ypad) + 2*ypad;
+		gtk_widget_set_size_request (GTK_WIDGET (mListView), 0, height);
+	}
+	else
+	{
+		GtkWidget *w;
+		w = glade_xml_get_widget(mGXml, "mime_ask_dialog_choice_label");
+		gtk_widget_hide(w);
+
+		w = glade_xml_get_widget(mGXml, "mime_ask_dialog_open");
+		gtk_widget_set_sensitive(w, FALSE);
+
+		gtk_widget_set_sensitive(mToggleButton, FALSE);
+	}	
+
+	GtkWidget *mimeIcon = glade_xml_get_widget(mGXml,
+						   "mime_ask_action_icon");
+	gtk_image_set_from_file(GTK_IMAGE(mimeIcon),
+				gnome_vfs_mime_get_icon(aMimeType));						     
+
+	description = gnome_vfs_mime_get_description(aMimeType);
+	if (!description) description = aMimeType;
+	
+	g_snprintf(ltext, 255, "<b>%s</b>", description);
+	label = glade_xml_get_widget(mGXml, "mime_ask_action_description");
+	gtk_label_set_markup(GTK_LABEL(label), ltext);
+
+	gtk_window_set_transient_for(GTK_WINDOW(mDialogWidget), 
+				     GTK_WINDOW(aParentWidget));
+
+	gtk_widget_show(mDialogWidget);
+}
+
+MimeAskActionDialog::~MimeAskActionDialog()
+{
+	if(mApps)
+		gnome_vfs_mime_application_list_free(mApps);
+
+	gtk_widget_destroy(mDialogWidget);
+	g_object_unref(G_OBJECT(mGXml));
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -685,32 +743,26 @@ nsresult NS_NewContentHandlerFactory(nsIFactory** aFactory)
 ////////////////////////////////////////////////////////////////////////////////
 
 extern "C" void
-mime_ask_dialog_save_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
+mime_ask_dialog_save_clicked_cb(GtkButton *button, MimeAskActionDialog *dialog)
 {
-	gtk_widget_hide (glade_xml_get_widget (dialog->gxml, 
-					       "mime_ask_action_dialog"));
+	gtk_widget_hide(glade_xml_get_widget(dialog->mGXml, 
+					     "mime_ask_action_dialog"));
 
 	nsresult rv;
-	nsCOMPtr<nsIHelperAppLauncher> aLauncher;
-	rv = dialog->mContentHandler->GetLauncher (getter_AddRefs(aLauncher));
+	nsCOMPtr<nsIHelperAppLauncher> launcher;
+	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
 	
-/* 	if (eel_gconf_get_boolean (CONF_DOWNLOADING_EXTERNAL_DOWNLOADER)
- 	    == DOWNLOADER_BUILTIN)*/
+ 	if (eel_gconf_get_boolean(CONF_DOWNLOADING_EXTERNAL_DOWNLOADER)
+ 	    == DOWNLOADER_BUILTIN)
  	{
-		aLauncher->SaveToDisk (nsnull,PR_FALSE);
+		launcher->SaveToDisk(nsnull,PR_FALSE);
 	}
-/*	else
+	else
 	{
-		aLauncher->Cancel ();
-		dialog->mContentHandler->LaunchExternalDownloader ();
-	}*/
+		launcher->Cancel();
+		dialog->mContentHandler->LaunchExternalDownloader();
+	}
 	
-	g_free (dialog->mime_type);
-	g_free (dialog->url);
-	g_free (dialog->scheme);
-	gtk_widget_destroy (glade_xml_get_widget (dialog->gxml, 
-						  "mime_ask_action_dialog"));
-	g_object_unref (G_OBJECT (dialog->gxml));
 	delete dialog;
 }
 
@@ -718,52 +770,75 @@ static void
 mime_ask_dialog_download_cancel (MimeAskActionDialog *dialog)
 {
 	nsresult rv;
-	nsCOMPtr<nsIHelperAppLauncher> aLauncher;
-	rv = dialog->mContentHandler->GetLauncher (getter_AddRefs(aLauncher));
+	nsCOMPtr<nsIHelperAppLauncher> launcher;
+	rv = dialog->mContentHandler->GetLauncher(getter_AddRefs(launcher));
 
-	aLauncher->Cancel ();
-	g_free (dialog->mime_type);
-	g_free (dialog->url);
+	launcher->Cancel();
 
-	gtk_widget_destroy (glade_xml_get_widget (dialog->gxml, 
-			    "mime_ask_action_dialog"));
-	g_object_unref (G_OBJECT (dialog->gxml));
+	delete dialog;
 }
 
 extern "C" void
 mime_ask_dialog_open_clicked_cb (GtkButton *button, MimeAskActionDialog *dialog)
 {
 	nsresult rv;
-	nsCOMPtr<nsIHelperAppLauncher> aLauncher;
-	rv = dialog->mContentHandler->GetLauncher (getter_AddRefs(aLauncher));
-	GnomeVFSMimeApplication *app;
+	nsCOMPtr<nsIHelperAppLauncher> launcher;
+	rv = dialog->mContentHandler->GetLauncher (getter_AddRefs(launcher));
+
+	GnomeVFSMimeApplication *app(NULL);
+	if (dialog->mApps)
+	{
+		GtkTreeSelection *selection =
+			gtk_tree_view_get_selection(GTK_TREE_VIEW(dialog->mListView));
+
+		GtkTreeModel *model;
+		GtkTreeIter iter;
+		if (gtk_tree_selection_get_selected (selection, &model, &iter))
+		{
+			GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
+			gint selected = atoi (gtk_tree_path_to_string (path));
+			gtk_tree_path_free (path);
+
+			app = static_cast<GnomeVFSMimeApplication *>
+				(g_list_nth_data (dialog->mApps, selected));
+		}
+	}
 
-	app = gnome_vfs_mime_get_default_application (dialog->mime_type);
 	if (app)
 	{
-		dialog->mContentHandler->SetHelperApp (app, FALSE);
+		dialog->mContentHandler->SetHelperApp(app, FALSE);
+		dialog->mContentHandler->FindHelperApp();
+		delete dialog;
 	}
 	else
 	{	
-		mime_ask_dialog_download_cancel (dialog);
-		galeon_embed_utils_nohandler_dialog_run (dialog->parent);
-		return;
+		mime_ask_dialog_download_cancel(dialog);
+		galeon_embed_utils_nohandler_dialog_run(dialog->mParent);
 	}
-	
-	dialog->mContentHandler->FindHelperApp ();
-	
-	g_free (dialog->mime_type);
-	g_free (dialog->url);
-	gtk_widget_destroy (glade_xml_get_widget (dialog->gxml, 
-						  "mime_ask_action_dialog"));
-	g_object_unref (G_OBJECT (dialog->gxml));
-	delete dialog;
 }
 
 extern "C" gint
-mime_ask_dialog_cancel_clicked_cb (GtkButton *button,
+mime_ask_dialog_cancel_clicked_cb(GtkButton *button,
 				   MimeAskActionDialog *dialog)
 {
-	mime_ask_dialog_download_cancel (dialog);
+	mime_ask_dialog_download_cancel(dialog);
 	return 0; /* FIXME: philipl, is this the right thing to return? */
 }
+
+extern "C" void
+mime_ask_dialog_toggle_helpers_toggled_cb(GtkToggleButton *button,
+					  MimeAskActionDialog *dialog)
+{
+	if (gtk_toggle_button_get_active(button))
+	{
+		gtk_widget_show(dialog->mScrolledView);
+		gtk_window_set_resizable(GTK_WINDOW(dialog->mDialogWidget),
+					 TRUE);
+	}
+	else
+	{
+		gtk_window_set_resizable(GTK_WINDOW(dialog->mDialogWidget),
+					 FALSE);
+		gtk_widget_hide(dialog->mScrolledView);
+	}
+}
