@@ -70,7 +70,12 @@
 #include "nsCWebBrowser.h"
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
+#if MOZILLA_SNAPSHOT > 7
+#include "nsIDOMPopupBlockedEvent.h"
+#include "nsIDOMNSDocument.h"
+#else
 #include "nsIDOMNSHTMLDocument.h"
+#endif
 #include "nsIDOMHTMLDocument.h"
 #include "nsIDOMHTMLCollection.h"
 #include "nsIDOMHTMLElement.h"
@@ -80,9 +85,18 @@
 #include "caps/nsIPrincipal.h"
 #include "nsIDeviceContext.h"
 #include "nsIPresContext.h"
+#include "nsITypeAheadFind.h"
+#include "nsSupportsPrimitives.h"
 #include "ContentHandler.h"
 
-GaleonWrapper::GaleonWrapper ()
+#include <nsIGenericFactory.h>
+
+GaleonWrapper::GaleonWrapper () :
+			mFaviconEventListener(new GDOMEventListener())
+#if MOZILLA_SNAPSHOT > 7
+			,mPopupEventListener(new GDOMPopupEventListener())
+#endif
+
 {
 }
 
@@ -90,6 +104,29 @@ GaleonWrapper::~GaleonWrapper ()
 {
 }
 
+static nsresult
+GetDOMEventTarget (nsIWebBrowser* aWebBrowser, nsIDOMEventTarget** aTarget)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIDOMWindow> domWin;
+	rv = aWebBrowser->GetContentDOMWindow (getter_AddRefs(domWin));
+	if (NS_FAILED(rv)) return rv;
+
+	nsCOMPtr<nsPIDOMWindow> piWin = do_QueryInterface(domWin, &rv);
+	if (NS_FAILED(rv)) return rv;
+
+	nsCOMPtr<nsIChromeEventHandler> eventHandler;
+	rv = piWin->GetChromeEventHandler (getter_AddRefs(eventHandler));
+	if (NS_FAILED(rv)) return rv;
+
+	nsCOMPtr<nsIDOMEventTarget> target = do_QueryInterface(eventHandler, &rv);
+	if (NS_FAILED(rv)) return rv;
+
+	NS_ADDREF(*aTarget = target.get());
+	return NS_OK;
+}
+
 nsresult GaleonWrapper::Init (GtkMozEmbed *mozembed)
 {
 	nsresult result;
@@ -98,6 +135,23 @@ nsresult GaleonWrapper::Init (GtkMozEmbed *mozembed)
 					 getter_AddRefs(mWebBrowser));
 	if (!mWebBrowser) return NS_ERROR_FAILURE;
 
+	// set up favicon listener
+	mFaviconEventListener->Init (GALEON_EMBED(mozembed));
+	result = ::GetDOMEventTarget (mWebBrowser, getter_AddRefs(mEventTarget));
+	if (NS_FAILED(result)) return result;
+	result = mEventTarget->AddEventListener(NS_LITERAL_STRING("DOMLinkAdded"),
+			                        mFaviconEventListener, PR_FALSE);
+	if (NS_FAILED(result)) g_warning ("AddFaviconEventListener failed!");
+
+#if MOZILLA_SNAPSHOT > 7
+	// set up popup block listener
+	mPopupEventListener->Init(GALEON_EMBED(mozembed));
+	result = mEventTarget->AddEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
+						mPopupEventListener, PR_FALSE);
+	if (NS_FAILED(result)) g_warning ("AddPopupEventListener failed!");
+#endif
+
+	// set up global history
 	nsCOMPtr<nsIDocShell> DocShell;
 	result = GetDocShell (getter_AddRefs(DocShell));
 	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
@@ -105,18 +159,6 @@ nsresult GaleonWrapper::Init (GtkMozEmbed *mozembed)
 	nsCOMPtr<nsIDocShellHistory> dsHistory = do_QueryInterface (DocShell);
 	if (!dsHistory) return NS_ERROR_FAILURE;
 
-	static NS_DEFINE_CID(kGlobalHistoryCID, GALEON_GLOBALHISTORY_CID);
-
-	nsCOMPtr<nsIFactory> GHFactory;
-	result = NS_NewGlobalHistoryFactory(getter_AddRefs(GHFactory));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
-
-	result = nsComponentManager::RegisterFactory(kGlobalHistoryCID,
-						     "Global history",
-						     NS_GLOBALHISTORY_CONTRACTID,
-						     GHFactory,
-						     PR_TRUE);
-
 	nsCOMPtr<nsIGlobalHistory> inst =  
 		do_GetService(NS_GLOBALHISTORY_CONTRACTID, &result);
 	return dsHistory->SetGlobalHistory(inst);
@@ -167,11 +209,7 @@ nsresult GaleonWrapper::Print (nsIPrintSettings *options, PRBool preview)
 	}
 	else
 	{
-#if MOZILLA_SNAPSHOT > 1
 		result = print->PrintPreview(options, nsnull, nsnull);
-#else
-		result = print->PrintPreview(options);
-#endif
 	}
 
 	return result;
@@ -249,6 +287,15 @@ nsresult GaleonWrapper::GetSHistory (nsISHistory **aSHistory)
 
 nsresult GaleonWrapper::Destroy ()
 {
+	mEventTarget->RemoveEventListener(NS_LITERAL_STRING("DOMLinkAdded"),
+					  mFaviconEventListener, PR_FALSE);
+
+#if MOZILLA_SNAPSHOT > 7
+	mEventTarget->RemoveEventListener(NS_LITERAL_STRING("DOMPopupBlocked"),
+					  mPopupEventListener, PR_FALSE);
+#endif
+
+	mEventTarget = nsnull;
       	mWebBrowser = nsnull;
 	mChromeNav = nsnull;
 	
@@ -469,12 +516,41 @@ nsresult GaleonWrapper::GetSHUrlAtIndex (PRInt32 index, nsCString &url)
 }
 
 nsresult GaleonWrapper::Find (const PRUnichar *search_string, 
+			      PRBool interactive,
 			      PRBool matchcase, PRBool search_backwards,
 			      PRBool search_wrap_around,
 			      PRBool search_for_entire_word,
 			      PRBool search_in_frames,
 			      PRBool *didFind)
 {
+	if (!interactive)
+	{
+		nsresult rv;
+		nsCOMPtr<nsITypeAheadFind> tAFinder
+			(do_GetService(NS_TYPEAHEADFIND_CONTRACTID, &rv));
+		if (NS_SUCCEEDED(rv))
+		{
+			nsCOMPtr<nsIDOMWindow> aFocusedWindow;
+			rv = GetFocusedDOMWindow(getter_AddRefs(aFocusedWindow));
+			if (NS_SUCCEEDED(rv))
+			{
+				nsSupportsInterfacePointerImpl windowPtr;
+				windowPtr.SetData(aFocusedWindow);
+
+				tAFinder->FindNext(search_backwards, &windowPtr);
+
+				nsCOMPtr<nsISupports> retValue;
+				rv = windowPtr.GetData(getter_AddRefs(retValue));
+				if (NS_SUCCEEDED(rv) && !retValue)
+				{
+					*didFind = PR_TRUE;
+					return NS_OK;
+				}
+			}
+		}
+
+	}
+
 	nsCOMPtr<nsIWebBrowserFind> finder (do_GetInterface(mWebBrowser));
 
 	finder->SetSearchString (search_string);
@@ -556,12 +632,12 @@ nsresult GaleonWrapper::GetMainDocumentUrl (nsCString &url)
 {
 	nsresult result;
 
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
 
-	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+	result = GetMainDOMDocument (getter_AddRefs(DOMDocument));
+	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
+	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIURI> uri;
@@ -574,12 +650,12 @@ nsresult GaleonWrapper::GetDocumentUrl (nsCString &url)
 {
         nsresult result;
 
-        nsCOMPtr<nsIDOMDocument> aDOMDocument;
+        nsCOMPtr<nsIDOMDocument> DOMDocument;
 
-        result = GetDOMDocument (getter_AddRefs(aDOMDocument));
-        if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+        result = GetDOMDocument (getter_AddRefs(DOMDocument));
+        if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
-        nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
+        nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
         if(!doc) return NS_ERROR_FAILURE;
 
         nsCOMPtr<nsIURI> uri;
@@ -594,62 +670,58 @@ nsresult GaleonWrapper::GetDocumentTitle (char **title)
 {
 	nsresult result;
 
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
 
-	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+	result = GetDOMDocument (getter_AddRefs(DOMDocument));
+	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
+	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
+#if MOZILLA_SNAPSHOT > 6
+	nsAutoString t;
+	doc->GetDocumentTitle(t);
+	*title = g_strdup (NS_ConvertUCS2toUTF8(t).get());
+#else
 	const nsString* t;
 	t = doc->GetDocumentTitle();
 
 	*title = g_strdup (NS_ConvertUCS2toUTF8(*t).get());
+#endif
 
 	return NS_OK;
 }
 
-nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
+nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest, 
+					PRBool back_history, 
+					PRBool forward_history, 
+					PRBool set_current)
 {
 	nsresult result;
-	int count,index;
-
-	nsCOMPtr<nsIDocShell> DocShell;
-	result = GetDocShell (getter_AddRefs(DocShell));
-	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsIWebNavigation> wn_src = do_QueryInterface (DocShell,
-							       &result);
-	if (!wn_src) return NS_ERROR_FAILURE;
+	PRInt32 count, index;
 	
 	nsCOMPtr<nsISHistory> h_src;
-	result = wn_src->GetSessionHistory (getter_AddRefs (h_src));
-	if (!NS_SUCCEEDED(result) || (!h_src)) return NS_ERROR_FAILURE;
+	result = GetSHistory (getter_AddRefs(h_src));
+	if (NS_FAILED(result) || !h_src) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDocShell> destDocShell;
-	result = dest->GetDocShell (getter_AddRefs(destDocShell));
-	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
+	h_src->GetCount (&count);
+	h_src->GetIndex (&index);
 
-	nsCOMPtr<nsIWebNavigation> wn_dest = do_QueryInterface (destDocShell,
-								&result);
-	if (!wn_dest) return NS_ERROR_FAILURE;
-	
 	nsCOMPtr<nsISHistory> h_dest;
-	result = wn_dest->GetSessionHistory (getter_AddRefs (h_dest));
+	result = dest->GetSHistory (getter_AddRefs (h_dest));
 	if (!NS_SUCCEEDED (result) || (!h_dest)) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsISHistoryInternal> hi_dest = do_QueryInterface (h_dest);
 	if (!hi_dest) return NS_ERROR_FAILURE;
 
-	h_src->GetCount (&count);
-	h_src->GetIndex (&index);
-
 	if (count) {
 		nsCOMPtr<nsIHistoryEntry> he;
 		nsCOMPtr<nsISHEntry> she;
 
-		for (PRInt32 i = 0; i < count; i++) {
+		for (PRInt32 i = (back_history ? 0 : index + 1); 
+		     i < (forward_history ? count : index + 1);
+		     i++) 
+		{
 
 			result = h_src->GetEntryAtIndex (i, PR_FALSE,
 							 getter_AddRefs (he));
@@ -663,15 +735,24 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
 			if (!NS_SUCCEEDED(result) || (!she))
 				return NS_ERROR_FAILURE;
 		}
-
-		result = wn_dest->GotoIndex(index);
-		if (!NS_SUCCEEDED(result)) return NS_ERROR_FAILURE;
+		
+		if (set_current)
+		{
+			nsCOMPtr<nsIDocShell> destDocShell;
+			result = dest->GetDocShell (getter_AddRefs(destDocShell));
+			if (NS_FAILED(result) || !destDocShell) return NS_ERROR_FAILURE;
+		
+			nsCOMPtr<nsIWebNavigation> wn_dest = do_QueryInterface (destDocShell, &result);
+			
+			result = wn_dest->GotoIndex(index);
+			if (!NS_SUCCEEDED(result)) return NS_ERROR_FAILURE;
+		}
 	}
 
 	return NS_OK;
 }
 
-nsresult GaleonWrapper::ForceCharacterSet (char *charset) 
+nsresult GaleonWrapper::ForceCharacterSet (const char *charset) 
 {
 	nsresult result;
 
@@ -728,71 +809,6 @@ nsresult GaleonWrapper::Paste(void)
 	return clipboard->Paste ();
 }
 
-nsresult GaleonWrapper::GetFaviconURL (nsCString &favicon_url)
-{
-	nsresult result;
-	PRUint32 links_count;
-
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
-	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
-
-	NS_NAMED_LITERAL_STRING(strname, "LINK");
-
-	nsCOMPtr<nsIDOMNodeList> aLinks;
-	result = aDOMDocument->GetElementsByTagName (strname, 
-						     getter_AddRefs (aLinks));
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-
-	result = aLinks->GetLength (&links_count);
-	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-
-	for (PRUint32 i = 0; i < links_count; i++)
-	{
-		nsCOMPtr<nsIDOMNode> aLink;
-		result = aLinks->Item (i, getter_AddRefs (aLink));
-		if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-
-		nsCOMPtr<nsIDOMElement> linkElement;
-		linkElement = do_QueryInterface (aLink);
-		if (!linkElement) return NS_ERROR_FAILURE;
-
-
-		NS_NAMED_LITERAL_STRING(attr_rel, "rel");
-		nsAutoString value;
-		result = linkElement->GetAttribute (attr_rel, value);
-		if (NS_FAILED(result)) return NS_ERROR_FAILURE;
-
-		if (value.Equals(NS_LITERAL_STRING("SHORTCUT ICON"),
-				 nsCaseInsensitiveStringComparator()) ||
-		    value.Equals(NS_LITERAL_STRING("ICON"),
-		    		 nsCaseInsensitiveStringComparator()))
-		{
-			NS_NAMED_LITERAL_STRING(attr_href, "href");
-			nsAutoString value;
-			result = linkElement->GetAttribute (attr_href, value);
-			if (NS_FAILED (result) || value.IsEmpty())
-				return NS_ERROR_FAILURE;
-			
-			nsCOMPtr<nsIDocument> doc = 
-				do_QueryInterface (aDOMDocument);
-			if(!doc) return NS_ERROR_FAILURE;
-			
-			nsCOMPtr<nsIURI> uri;
-			result = doc->GetDocumentURL(getter_AddRefs(uri));
-			if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-
-			const nsACString &link = NS_ConvertUCS2toUTF8(value);
-			result = uri->Resolve (link, favicon_url);
-			if (NS_FAILED (result)) return NS_ERROR_FAILURE;
-			
-			return NS_OK;
-		}
-	}
-
-	return NS_ERROR_FAILURE;
-}
-
 nsresult GaleonWrapper::GetStyleSheets(nsIDOMStyleSheetList** list)
 {
 	nsresult result;
@@ -814,7 +830,6 @@ nsresult GaleonWrapper::LoadOverrideStyleSheet(char *css,
 					       nsIStyleSheet **return_sheet)
 {
 	nsresult result;
-	PRBool completed;
 
 	/* catch necessary stuff */
 	nsCOMPtr<nsIDocShell> DocShell;
@@ -841,7 +856,7 @@ nsresult GaleonWrapper::LoadOverrideStyleSheet(char *css,
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
 	nsICSSStyleSheet *sheet;
-	result = loader->LoadAgentSheet(uri, sheet, completed, nsnull);
+	result = loader->LoadAgentSheet(uri, &sheet);
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
 	/* catch stylesheet stuff and apply by appending it as a override
@@ -874,11 +889,11 @@ nsresult GaleonWrapper::RemoveOverrideStyleSheet(nsIStyleSheet *remove)
 	nsresult result;
 
 	/* get all necessary stuff */
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
-	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
+	result = GetMainDOMDocument (getter_AddRefs(DOMDocument));
+	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDocument> doc = do_QueryInterface(aDOMDocument);
+	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
 	/* catch presshell */
@@ -916,26 +931,26 @@ nsresult GaleonWrapper::GetLinkInterfaceItems (GList **list)
 		NULL /* terminator, must be last */
 	};
 
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
-	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
+	result = GetMainDOMDocument (getter_AddRefs(DOMDocument));
+	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
 	/* get list of link elements*/
 	NS_NAMED_LITERAL_STRING(strname, "LINK");
 
-	nsCOMPtr<nsIDOMNodeList> aLinks;
+	nsCOMPtr<nsIDOMNodeList> links;
 	result = aDOMDocument->GetElementsByTagName (strname, 
-						     getter_AddRefs (aLinks));
+						     getter_AddRefs (links));
 	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 
-	result = aLinks->GetLength (&links_count);
+	result = links->GetLength (&links_count);
 	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 
 	for (PRUint32 i = 0; i < links_count; i++)
 	{
 		/* get to the link element */
-		nsCOMPtr<nsIDOMNode> aLink;
-		result = aLinks->Item (i, getter_AddRefs (aLink));
+		nsCOMPtr<nsIDOMNode> link;
+		result = links->Item (i, getter_AddRefs (link));
 		if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 
 		nsCOMPtr<nsIDOMElement> linkElement;
@@ -1108,13 +1123,18 @@ nsresult GaleonWrapper::GetLastModified (gchar **ret)
 {
 	nsresult result;
 
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
 
-	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+	result = GetDOMDocument (getter_AddRefs(DOMDocument));
+	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDOMNSHTMLDocument> doc = do_QueryInterface(aDOMDocument);
+#if MOZILLA_SNAPSHOT > 7
+	nsCOMPtr<nsIDOMNSDocument> doc = do_QueryInterface(DOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
+#else
+	nsCOMPtr<nsIDOMNSHTMLDocument> doc = do_QueryInterface(DOMDocument);
+	if(!doc) return NS_ERROR_FAILURE;
+#endif
 
 	nsAutoString value;
 	doc->GetLastModified(value);
@@ -1130,12 +1150,12 @@ nsresult GaleonWrapper::GetImages (GList **ret)
 	nsresult result;
 	GHashTable *hash = g_hash_table_new (g_str_hash, g_str_equal);
 
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
 
-	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+	result = GetDOMDocument (getter_AddRefs(DOMDocument));
+	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDOMHTMLDocument> doc = do_QueryInterface(aDOMDocument);
+	nsCOMPtr<nsIDOMHTMLDocument> doc = do_QueryInterface(DOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDOMHTMLCollection> col;
@@ -1201,12 +1221,12 @@ nsresult GaleonWrapper::GetForms (GList **ret)
 #ifdef NOT_PORTED
 	nsresult result;
 
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
 
-	result = GetDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+	result = GetDOMDocument (getter_AddRefs(DOMDocument));
+	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDOMHTMLDocument> doc = do_QueryInterface(aDOMDocument);
+	nsCOMPtr<nsIDOMHTMLDocument> doc = do_QueryInterface(DOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDOMHTMLCollection> col;
@@ -1271,18 +1291,18 @@ nsresult GaleonWrapper::GetLinks (GList **ret)
 #ifdef NOT_PORTED
 	nsresult result;
 
-	nsCOMPtr<nsIDOMDocument> aDOMDocument;
-	result = GetMainDOMDocument (getter_AddRefs(aDOMDocument));
-	if (NS_FAILED(result) || !aDOMDocument) return NS_ERROR_FAILURE;
+	nsCOMPtr<nsIDOMDocument> DOMDocument;
+	result = GetMainDOMDocument (getter_AddRefs(DOMDocument));
+	if (NS_FAILED(result) || !DOMDocument) return NS_ERROR_FAILURE;
 
 	/* first, get a list of <link> elements */
 	PRUint32 links_count;
 
 	NS_NAMED_LITERAL_STRING(strname, "LINK");
 
-	nsCOMPtr<nsIDOMNodeList> aLinks;
-	result = aDOMDocument->GetElementsByTagName (strname, 
-						     getter_AddRefs (aLinks));
+	nsCOMPtr<nsIDOMNodeList> links;
+	result = DOMDocument->GetElementsByTagName (strname, 
+						     getter_AddRefs (links));
 	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 
 	result = aLinks->GetLength (&links_count);
@@ -1290,12 +1310,12 @@ nsresult GaleonWrapper::GetLinks (GList **ret)
 
 	for (PRUint32 i = 0; i < links_count; i++)
 	{
-		nsCOMPtr<nsIDOMNode> aLink;
-		result = aLinks->Item (i, getter_AddRefs (aLink));
+		nsCOMPtr<nsIDOMNode> link;
+		result = links->Item (i, getter_AddRefs (link));
 		if (NS_FAILED (result)) continue;
 
 		nsCOMPtr<nsIDOMElement> linkElement;
-		linkElement = do_QueryInterface (aLink);
+		linkElement = do_QueryInterface (link);
 		if (!linkElement) continue;
 
 		NS_NAMED_LITERAL_STRING(attr_href, "href");
@@ -1474,3 +1494,108 @@ nsresult GaleonWrapper::PopTargetDocument ()
 
 	return NS_OK;
 }
+
+////////////////////////////////////////////////////////////////////////////
+NS_IMPL_ISUPPORTS1(GDOMEventListener, nsIDOMEventListener)
+
+GDOMEventListener::GDOMEventListener() : mEmbed(0)
+{
+}
+
+GDOMEventListener::~GDOMEventListener()
+{
+}
+
+nsresult
+GDOMEventListener::Init(GaleonEmbed *aEmbed)
+{
+	mEmbed = aEmbed;
+	return NS_OK;
+}
+
+static nsresult
+ResolveURI (nsIDocument* doc, const nsAString& relative_uri,
+	    nsACString& full_uri)
+{
+	g_return_val_if_fail (doc != NULL, NS_ERROR_FAILURE);
+
+	nsresult rv;
+
+	nsCOMPtr<nsIURI> docUri;
+	rv = doc->GetDocumentURL (getter_AddRefs(docUri));
+	if (NS_FAILED(rv)) return rv;
+	
+	const nsACString &link = NS_ConvertUCS2toUTF8 (relative_uri);
+	return docUri->Resolve (link, full_uri);
+}
+
+NS_IMETHODIMP
+GDOMEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIDOMEventTarget> target;
+	rv = aDOMEvent->GetTarget(getter_AddRefs(target));
+	if (NS_FAILED(rv)) return rv;
+
+	nsCOMPtr<nsIDOMElement> elem = do_QueryInterface(target, &rv);
+	if (NS_FAILED(rv)) return rv;
+
+	nsAutoString value;
+	rv = elem->GetAttribute(NS_LITERAL_STRING("rel"), value);
+	if (NS_FAILED(rv)) return rv;
+
+	if (value.Equals(NS_LITERAL_STRING("SHORTCUT ICON"),
+	                 nsCaseInsensitiveStringComparator()) ||
+	    value.Equals(NS_LITERAL_STRING("ICON"), 
+		         nsCaseInsensitiveStringComparator()))
+	{
+		rv = elem->GetAttribute (NS_LITERAL_STRING("href"), value);
+		if (NS_FAILED(rv)) return rv;
+		if (value.IsEmpty()) return NS_ERROR_FAILURE;
+
+		nsCOMPtr<nsIDOMDocument> domDoc;
+		rv = elem->GetOwnerDocument (getter_AddRefs(domDoc));
+		if (NS_FAILED(rv)) return rv;
+
+		nsCOMPtr<nsIDocument> doc = do_QueryInterface (domDoc, &rv);
+		if (NS_FAILED(rv)) return rv;
+
+		nsCAutoString favicon;
+		rv = ::ResolveURI (doc, value, favicon);
+		if (NS_SUCCEEDED(rv))
+		{
+			g_signal_emit_by_name (mEmbed, "ge_favicon", favicon.get());
+		}
+	}
+
+	return NS_OK;
+}
+
+//------------------------------------------------------------------------------
+#if MOZILLA_SNAPSHOT > 7
+NS_IMPL_ISUPPORTS1(GDOMPopupEventListener, nsIDOMEventListener)
+
+NS_IMETHODIMP
+GDOMPopupEventListener::HandleEvent (nsIDOMEvent* aDOMEvent)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsIDOMPopupBlockedEvent> popupEvent =
+		do_QueryInterface(aDOMEvent, &rv);
+	if (NS_FAILED(rv) || !popupEvent) return NS_ERROR_FAILURE;
+
+	nsCOMPtr<nsIURI> requesterURI;
+	rv = popupEvent->GetRequestingWindowURI(getter_AddRefs(requesterURI));
+	if (NS_FAILED(rv) || !popupEvent) return NS_ERROR_FAILURE;
+
+	nsCAutoString spec;
+	rv = requesterURI->GetSpec(spec);
+	if (NS_FAILED(rv) || spec.IsEmpty()) return NS_ERROR_FAILURE;
+
+	g_signal_emit_by_name (mEmbed, "ge_popupblocked", spec.get());
+
+	return rv;
+}
+#endif
+
