@@ -17,44 +17,22 @@
  */
 
 #include "galeon-config.h"
-#include "mozilla-embed-shell.h"
 
 #include "nsCOMPtr.h"
 #include "nsISupportsArray.h"
-#include "nsIFactory.h"
 #include "nsIServiceManager.h"
-#include "nsXPComFactory.h"
 #include "nsString.h"
 #include "nsReadableUtils.h"
-#include "nsIGlobalHistory.h"
-#include "nsIBrowserHistory.h"
 #include "nsIRequestObserver.h"
 
+#include "GlobalHistory.h"
+
 /**
  * class GlobalHistory: 
  *
  */
-class MozGlobalHistory: public nsIGlobalHistory,
-	public nsIBrowserHistory
-{
-	public:
-		MozGlobalHistory ();
-  		virtual ~MozGlobalHistory();
 
-		NS_DECL_ISUPPORTS
-		NS_DECL_NSIGLOBALHISTORY
-		NS_DECL_NSIBROWSERHISTORY
-
-	private:
-		GlobalHistory *mGlobalHistory;
-};
-
-NS_IMPL_ADDREF(MozGlobalHistory)
-NS_IMPL_RELEASE(MozGlobalHistory)
-NS_INTERFACE_MAP_BEGIN(MozGlobalHistory)
-  NS_INTERFACE_MAP_ENTRY(nsIGlobalHistory)
-  NS_INTERFACE_MAP_ENTRY(nsIBrowserHistory)
-NS_INTERFACE_MAP_END
+NS_IMPL_ISUPPORTS2(MozGlobalHistory, nsIGlobalHistory, nsIBrowserHistory)
 
 MozGlobalHistory::MozGlobalHistory ()
 {
@@ -131,7 +109,8 @@ NS_IMETHODIMP MozGlobalHistory::GetCount(PRUint32 *aCount)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
- 
+
+#if MOZILLA_SNAPSHOT <= 8 
 /* void startBatchUpdate (); */
 NS_IMETHODIMP MozGlobalHistory::StartBatchUpdate()
 {
@@ -143,39 +122,10 @@ NS_IMETHODIMP MozGlobalHistory::EndBatchUpdate()
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
+#endif
 
 /* void markPageAsTyped (in string url); */
 NS_IMETHODIMP MozGlobalHistory::MarkPageAsTyped(const char *url)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
-
-#if MOZILLA_SNAPSHOT == 5
-/* Described by mozilla.org as a temporary ugly hack. We will never need to
- * implement it. It is here to allow compilation.
- */
-/* void outputReferrerURL (in string aURL, in string aReferrer); */
-NS_IMETHODIMP MozGlobalHistory::OutputReferrerURL(const char *aURL, const char *aReferrer)
-{
-    return NS_ERROR_NOT_IMPLEMENTED;
-}
-#endif
-
-NS_DEF_FACTORY (MozGlobalHistory, MozGlobalHistory);
-
-nsresult NS_NewGlobalHistoryFactory(nsIFactory** aFactory)
-{
-	NS_ENSURE_ARG_POINTER(aFactory);
-	*aFactory = nsnull;
-
-	nsMozGlobalHistoryFactory *result = new nsMozGlobalHistoryFactory;
-	if (result == NULL)
-	{
-		return NS_ERROR_OUT_OF_MEMORY;
-	}
-    
-	NS_ADDREF(result);
-	*aFactory = result;
-
-	return NS_OK;
-}
