@@ -396,14 +396,14 @@ embed_mouse_click_cb  (GaleonEmbed *embed,
 	galeon_embed_event_get_context (event, &context);
 	g_assert (IS_GALEON_EMBED_EVENT(event));
 
-	guint button;	
+	guint button, modifier;
 	galeon_embed_event_get_mouse_button (event, &button);
+	galeon_embed_event_get_modifier(event, &modifier);
 
 	const GValue *targetValue;
 	galeon_embed_event_get_property (event, "link_target", &targetValue);
 
-	if (button == 0 && (context & EMBED_CONTEXT_LINK) &&
-	    (galeon_embed_event_get_modifier(event) == 0) &&
+	if (button == 0 && (context & EMBED_CONTEXT_LINK) && (modifier == 0) &&
 	    (nsCRT::strcmp(g_value_get_string(targetValue), "_content") == 0))
 	{
 		const GValue *linkValue;
