@@ -217,6 +217,9 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event, Mozill
 static gint
 mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event, 
 				 MozillaEmbed *membed);
+static gint
+mozilla_embed_dom_key_press_cb (GtkMozEmbed *embed, gpointer dom_event,
+				MozillaEmbed *membed);
 static void
 mozilla_embed_size_to_cb (GtkMozEmbed *embed, gint width, gint height, MozillaEmbed *membed);
 static void
@@ -250,6 +253,7 @@ signal_connections[] =
         { "destroy_browser", (void *) mozilla_embed_destroy_brsr_cb      },
         { "dom_mouse_click", (void *) mozilla_embed_dom_mouse_click_cb   },
 	{ "dom_mouse_down",  (void *) mozilla_embed_dom_mouse_down_cb    },
+	{ "dom_key_press",   (void *) mozilla_embed_dom_key_press_cb	 },
         { "size_to",         (void *) mozilla_embed_size_to_cb           },
         { "new_window",      (void *) mozilla_embed_new_window_cb        },
         { "security_change", (void *) mozilla_embed_security_change_cb   },
@@ -1401,25 +1405,30 @@ mozilla_embed_dom_mouse_click_cb (GtkMozEmbed *embed, gpointer dom_event,
 	nsresult result;
 
 	wrapper = MOZILLA_EMBED(membed)->priv->wrapper;
-	g_return_val_if_fail (wrapper != NULL, G_FAILED);
+	g_return_val_if_fail(wrapper != NULL, G_FAILED);
 	
-	info = galeon_embed_event_new ();
+	info = galeon_embed_event_new();
 	
-	event_context.Init ((nsIDOMEvent*)dom_event, wrapper);
-        result = event_context.GetMouseEventInfo (info);
+	event_context.Init((nsIDOMEvent*)dom_event, wrapper);
+        result = event_context.GetMouseEventInfo(info);
+	if (NS_SUCCEEDED(result))
+	{
+		nsCOMPtr<nsIDOMDocument> domDoc;
+		result = event_context.GetTargetDocument(getter_AddRefs(domDoc));
+		if (NS_SUCCEEDED(result))
+		{
+			result = wrapper->PushTargetDocument(domDoc);
+			if (NS_SUCCEEDED(result))
+			{
+				g_signal_emit_by_name(membed, "ge_dom_mouse_click", 
+						      info, &return_value); 
+				wrapper->PopTargetDocument();
+			}
+		}
 
-        nsCOMPtr<nsIDOMDocument> domDoc;
-        result = event_context.GetTargetDocument (getter_AddRefs(domDoc));
-        if (NS_SUCCEEDED(result))
-        {
-                result = wrapper->PushTargetDocument (domDoc);
-        }
+	}
 
-	g_signal_emit_by_name (membed, "ge_dom_mouse_click", info, &return_value); 
-	
-	wrapper->PopTargetDocument ();
-	
-	g_object_unref (info);
+	g_object_unref(info);
 	
 	return return_value;
 }
@@ -1463,6 +1472,14 @@ mozilla_embed_dom_mouse_down_cb (GtkMozEmbed *embed, gpointer dom_event,
 	return return_value;
 }
 
+static gint
+mozilla_embed_dom_key_press_cb (GtkMozEmbed *embed, gpointer dom_event,
+				MozillaEmbed *membed)
+{
+	g_print("Hello keypress\n");
+	return false;
+}
+
 static void
 mozilla_embed_size_to_cb (GtkMozEmbed *embed, gint width, gint height, 
 			  MozillaEmbed *membed)
