@@ -17,6 +17,7 @@
  */
 
 #include "galeon-config.h"
+#include "galeon-embed-shell.h"
 #include "mozilla-notifiers.h"
 #include "eel-gconf-extensions.h"
 #include "mozilla-prefs.h"
@@ -94,6 +95,24 @@ mozilla_default_font_notifier(GConfClient *client,
 			      GConfEntry *entry,
 			      gpointer user_data);
 
+static void
+mozilla_proxy_mode_notifier (GConfClient *client,
+			     guint cnxn_id,
+			     GConfEntry *entry,
+			     char *pref);
+
+static void
+mozilla_proxy_ignore_notifier (GConfClient *client,
+			     guint cnxn_id,
+			     GConfEntry *entry,
+			     char *pref);
+
+static void
+mozilla_proxy_autoconfig_notifier (GConfClient *client,
+			           guint cnxn_id,
+			           GConfEntry *entry,
+			           char *pref);
+
 static void
 mozilla_user_agent_notifier(GConfClient *client,
 			    guint cnxn_id,
@@ -134,7 +153,6 @@ conversion_table [] =
 	{ CONF_FILTERING_JAVA_ENABLED, BOOL_PREF, "security.enable_java"},
 	{ CONF_FILTERING_JAVASCRIPT_ENABLED, BOOL_PREF, "javascript.enabled"},
 	{ CONF_FILTERING_IMAGE_LOADING_TYPE, INT_PREF, "network.image.imageBehavior"},
-	{ CONF_NETWORK_PROXY_MODE, INT_PREF, "network.proxy.type"},
 	{ CONF_RENDERING_BG_COLOR, STRING_PREF, "browser.display.background_color"},
 	{ CONF_RENDERING_TEXT_COLOR, STRING_PREF, "browser.display.foreground_color"},
 	{ CONF_RENDERING_UNVISITED_LINKS, STRING_PREF, "browser.anchor_color"},
@@ -149,7 +167,6 @@ conversion_table [] =
 	{ CONF_NETWORK_FTP_PROXY_PORT, INT_PREF, "network.proxy.ftp_port"},
 	{ CONF_NETWORK_SSL_PROXY_PORT, INT_PREF, "network.proxy.ssl_port"},
 	{ CONF_NETWORK_SOCKS_PROXY_PORT, INT_PREF, "network.proxy.socks_port"},
-	{ CONF_NETWORK_NO_PROXIES_FOR, STRING_PREF, "network.proxy.no_proxies_on"},
 	{ CONF_NETWORK_PROXY_KA, BOOL_PREF, "network.http.proxy.keep-alive"},
 	{ CONF_NETWORK_MEMORY_CACHE, INT_PREF, "browser.cache.memory.capacity"},
 	{ CONF_NETWORK_DISK_CACHE, INT_PREF, "browser.cache.disk.capacity"},
@@ -208,6 +225,12 @@ custom_notifiers [] =
 	  (GConfClientNotifyFunc) mozilla_default_font_notifier },
 	{ CONF_NETWORK_SOCKS_PROXY_VERSION, 
 	  (GConfClientNotifyFunc) mozilla_socks_version_notifier },
+	{ CONF_NETWORK_PROXY_MODE,
+	  (GConfClientNotifyFunc) mozilla_proxy_mode_notifier },
+	{ CONF_NETWORK_PROXY_AUTO_URL,
+	  (GConfClientNotifyFunc) mozilla_proxy_autoconfig_notifier },
+	{ CONF_NETWORK_PROXY_IGNORE_HOSTS,
+	  (GConfClientNotifyFunc) mozilla_proxy_ignore_notifier },
 	{NULL, NULL}
 };
 
@@ -224,6 +247,57 @@ mozilla_font_size_notifier (GConfClient *client,
 	mozilla_prefs_set_int (key, gconf_value_get_int(entry->value));
 }
 
+static void
+mozilla_proxy_mode_notifier (GConfClient *client,
+		             guint cnxn_id,
+		             GConfEntry *entry,
+		             char *pref)
+{
+	const char *mode;
+	int mozilla_mode = 0;
+	
+	mode = gconf_value_get_string(entry->value);
+	
+	if (strcmp (mode, "manual") == 0)
+	{
+		mozilla_mode = 1;
+	}
+	else if (strcmp (mode, "auto") == 0)
+	{
+		mozilla_mode = 2;
+	}
+
+	mozilla_prefs_set_int ("network.proxy.type", mozilla_mode);
+}
+
+static void
+mozilla_proxy_ignore_notifier (GConfClient *client,
+		             guint cnxn_id,
+		             GConfEntry *entry,
+		             char *pref)
+{
+	GSList *hosts, *pos;
+	gchar **strings, *mozilla_ignore_list;
+	int i;
+
+	hosts = gconf_value_get_list (entry->value);
+
+	strings = g_new (gchar*, g_slist_length (hosts) + 1);
+	for (pos = hosts, i = 0; pos; pos = g_slist_next (pos), ++i)
+	{
+		strings[i] = (gchar*) gconf_value_get_string (
+				(GConfValue*) (pos->data));
+	}
+	strings[i] = NULL;
+
+	mozilla_ignore_list = g_strjoinv (", ", strings);
+	mozilla_prefs_set_string ("network.proxy.no_proxies_on",
+			mozilla_ignore_list);
+
+	g_free (mozilla_ignore_list);
+	g_free (strings);
+}
+
 static void
 mozilla_font_notifier (GConfClient *client,
 		       guint cnxn_id,
@@ -237,9 +311,57 @@ mozilla_font_notifier (GConfClient *client,
 	mozilla_prefs_set_string (key, gconf_value_get_string(entry->value));
 }
 
+static void
+mozilla_proxy_autoconfig_notifier (GConfClient *client,
+		       	           guint cnxn_id,
+		       		   GConfEntry *entry,
+		       		   char *pref)
+{
+	galeon_embed_shell_load_proxy_autoconf 
+		(embed_shell, gconf_value_get_string(entry->value));
+}
+
+static void
+add_notification_and_notify (GConfClient		*client,
+			     const char 		*key,
+			     GConfClientNotifyFunc	 func,
+			     gpointer			 user_data)
+{
+	GConfEntry *entry;
+	GError *error = NULL;
+	guint cnxn_id;
+
+	cnxn_id = gconf_client_notify_add (client, key, func, user_data, NULL, &error);
+	if (eel_gconf_handle_error (&error))
+	{
+		if (cnxn_id != EEL_GCONF_UNDEFINED_CONNECTION)
+		{
+			gconf_client_notify_remove (client, cnxn_id);
+		}
+		return;
+	}
+
+	mozilla_notifiers = g_list_append (mozilla_notifiers,
+			                   GUINT_TO_POINTER (cnxn_id));
+
+	entry = gconf_client_get_entry (client, key, NULL, TRUE, &error);
+	if (eel_gconf_handle_error (&error))
+	{
+		return;
+	}
+	g_return_if_fail (entry != NULL);
+
+	if (entry->value != NULL)
+	{
+		func (client, cnxn_id, entry, user_data);
+	}
+	gconf_entry_free (entry);
+}
+
 void 
 mozilla_notifiers_init(MozillaEmbedShell *shell) 
 {
+	GConfClient *client = eel_gconf_client_get_global ();
 	int i;
 
 	for (i = 0; conversion_table[i].gconf_key != NULL; i++)
@@ -261,20 +383,21 @@ mozilla_notifiers_init(MozillaEmbedShell *shell)
 
 		g_assert (func != NULL);
 		
-		galeon_notification_add(
+		add_notification_and_notify (
+			client,
 			conversion_table[i].gconf_key,
 			func,
-			(gpointer)conversion_table[i].mozilla_key,
-			&mozilla_notifiers);
+			(gpointer)conversion_table[i].mozilla_key);
+
 	}
 
 	for (i = 0; custom_notifiers[i].gconf_key != NULL; i++)
 	{
-			galeon_notification_add(
-				custom_notifiers[i].gconf_key,
-				custom_notifiers[i].func,
-				(gpointer)shell,
-				&mozilla_notifiers);
+		add_notification_and_notify (
+			client,
+			custom_notifiers[i].gconf_key,
+			custom_notifiers[i].func,
+			(gpointer)shell);
 	}
 
 	/* fonts notifiers */
@@ -340,40 +463,6 @@ mozilla_notifiers_free (void)
 	g_list_free (font_infos);
 }
 
-void 
-mozilla_notifiers_set_defaults(void) 
-{
-	GConfClient* client = eel_gconf_client_get_global();
-	GConfValue* value;
-	int i;
-
-	for (i = 0; conversion_table[i].gconf_key != NULL; i++)
-	{
-		value = gconf_client_get 
-			(client, conversion_table[i].gconf_key, NULL);
-		if (value)
-		{
-			gconf_client_set (client, 
-					  conversion_table[i].gconf_key,
-					  value, NULL);
-			gconf_value_free (value);
-		}
-	}
-
-	for (i = 0; custom_notifiers[i].gconf_key != NULL; i++)
-	{
-		value = gconf_client_get 
-			(client, custom_notifiers[i].gconf_key, NULL);
-		if (value)
-		{
-			gconf_client_set (client, 
-					  custom_notifiers[i].gconf_key,
-					  value, NULL);
-			gconf_value_free (value);
-		}
-	}
-}
-	
 /**
  * generic_mozilla_string_notify: update mozilla pref to match galeon prefs.
  * 	user_data should match the mozilla key
@@ -656,7 +745,7 @@ mozilla_prefs_set_user_agent ()
                 }
                 
                 default_user_agent = g_strdup_printf
-                        ("Mozilla/5.0 Galeon/" VERSION " (X11; %s; U;) Gecko/%d", 
+                        ("Mozilla/5.0 (X11; U; %s) Gecko/%d Galeon/" VERSION, 
                          system,
                          NS_BUILD_ID/100);
 
