@@ -39,8 +39,8 @@ static gboolean
 galeon_main_automation_init (void);
 static gint
 galeon_main_translate_url_arguments (poptContext context, gchar ***urls);
-static gboolean
-galeon_main_start (gpointer data);
+static void
+galeon_main_start (void);
 
 /* FIXME */
 GnomeProgram *program;
@@ -61,7 +61,6 @@ static gboolean open_as_nautilus_view = FALSE;
 static BonoboObject *automation_object;
 static gint n_urls; 
 static gchar **url;
-static gboolean first_instance;
 
 /* command line argument parsing structure */
 static struct poptOption popt_options[] =
@@ -141,9 +140,7 @@ main (int argc, char *argv[])
 	/* load arguments that aren't regular options (urls to load) */
         n_urls = galeon_main_translate_url_arguments (context, &url);
 
-	first_instance = galeon_main_automation_init ();
-
-	if (first_instance)
+	if (galeon_main_automation_init ())
 	{
 		gnome_vfs_init ();
 		
@@ -153,10 +150,12 @@ main (int argc, char *argv[])
 		
 		galeon_shell_new ();
 		
-		g_idle_add ((GSourceFunc) galeon_main_start, NULL);
-
+		galeon_main_start ();
+		
 		galeon_automation_complete_initialization ();
-
+		
+		g_object_unref (G_OBJECT(galeon_shell));
+		
 		bonobo_main ();
 		
 		gnome_vfs_shutdown ();
@@ -165,8 +164,8 @@ main (int argc, char *argv[])
 	return 0;
 }
 
-static gboolean
-galeon_main_start (gpointer data)
+static void
+galeon_main_start (void)
 {
 	GNOME_GaleonAutomation gaserver;
 	int i;
@@ -258,12 +257,6 @@ galeon_main_start (gpointer data)
 		}
 	}
 
-	/* Unref so it will exit if no more used */
-	if (first_instance)
-	{
-		g_object_unref (G_OBJECT(galeon_shell));
-	}
-
 	if (gaserver)
 	{
 		bonobo_object_release_unref (gaserver, &corba_env);
@@ -274,7 +267,6 @@ galeon_main_start (gpointer data)
 #if GTK_CHECK_VERSION(2,2,0)
 	gdk_notify_startup_complete ();
 #endif 
-	return FALSE;
 }
 
 static gboolean
@@ -294,7 +286,7 @@ galeon_main_automation_init (void)
 	}
 	else
 	{
-		galeon_main_start (NULL);
+		galeon_main_start ();
 		g_message (_("Galeon already running, using existing process"));
 		return FALSE;
 	}
