@@ -371,9 +371,10 @@ gnv_embed_dom_mouse_down_cb (GaleonEmbed *embed,
 		switch (action) 
 		{
 		case 0: /* bookmarks context menu */
-		case 1: /* paste url */
+		case 1: /* paste url in new tab */
 		case 2: /* go back */
 		case 3: /* gestures */
+		case 6: /* paste url in current tab */
 			
 			/* fallback */
 			
