@@ -29,11 +29,13 @@
 #include "galeon-macros.h"
 #include "galeon-shell.h"
 #include "galeon-embed-popup-bw.h"
+#include "galeon-embed-utils.h"
 #include "eel-gconf-extensions.h"
 #include "prefs-strings.h"
 #include "galeon-embed-prefs.h"
 #include "galeon-embed-autoscroller.h"
 #include "galeon-embed-manual-scroller.h"
+#include "gul-download.h"
 #include "gul-gestures.h"
 
 #include <bonobo/bonobo-i18n.h>
@@ -55,9 +57,12 @@ struct GaleonTabPrivate
 	GaleonWindow *window;
 	gboolean is_active;
 	TabLoadStatus load_status;
+	char *link_message;
+	char *js_message;
 	char status_message[255];
 	char *title;
 	char *location;
+	char *blocked_popup_uri;
 	int load_percent;
 	gboolean visibility;
 	int cur_requests;
@@ -130,10 +135,11 @@ galeon_tab_zoom_changed_cb (GaleonEmbed *embed, gint zoom,
 			    GaleonTab *tab);
 
 static void
-galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTab *tab);
+galeon_tab_popup_blocked_cb(GaleonEmbed *embed, const char *uri,
+			    GaleonTab *tab);
 
 static void
-galeon_tab_gesture_cancelled_cb (GulGestures *ges, GaleonTab *tab);
+galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTab *tab);
 
 static GObjectClass *parent_class = NULL;
 
@@ -177,6 +183,22 @@ galeon_tab_class_init (GaleonTabClass *klass)
         object_class->finalize = galeon_tab_finalize;
 }
 
+static void
+galeon_tab_parent_set_cb (GtkWidget *widget, GtkWidget *previous_parent, 
+			  GaleonTab *tab)
+{
+	GtkWidget *toplevel;
+
+	/* FIXME maybe we dont need to set the tab parent explicitly
+	 * anymore with this callback taking care of it */
+	
+	if (widget->parent == NULL) return;
+
+	toplevel = gtk_widget_get_toplevel (widget);
+
+	tab->priv->window = GALEON_WINDOW (toplevel);
+}
+
 static void
 galeon_tab_embed_destroy_cb (GtkWidget *widget, GaleonTab *tab)
 {
@@ -186,6 +208,35 @@ galeon_tab_embed_destroy_cb (GtkWidget *widget, GaleonTab *tab)
 	g_object_unref (tab);
 }
 
+static void
+galeon_tab_favicon_cb (GaleonEmbed *embed, const char *favicon_url,
+		       GaleonTab *tab)
+{
+	char *url1 = NULL, *url2 = NULL;
+	gboolean download;
+	gresult result;
+
+	result = galeon_embed_get_location (embed, TRUE, FALSE, &url1);
+	if (result != G_OK) return;
+
+	result = galeon_embed_get_location (embed, TRUE, TRUE, &url2);
+	if (result != G_OK) goto out;
+
+	download = (url1 != NULL && url2 != NULL && strcmp (url1, url2) == 0);
+	if (download)
+	{
+		GaleonFaviconCache *cache;
+		cache = galeon_shell_get_favicon_cache (galeon_shell);
+		galeon_favicon_cache_insert_from_url (cache,
+				                      url1,
+						      favicon_url);
+	}
+
+out:
+	g_free (url1);
+	g_free (url2);
+}
+
 static void
 galeon_tab_init (GaleonTab *tab)
 {
@@ -200,6 +251,8 @@ galeon_tab_init (GaleonTab *tab)
 
 	tab->priv->window = NULL;
 	tab->priv->is_active = FALSE;
+	tab->priv->link_message = g_strdup ("");
+	tab->priv->js_message = g_strdup ("");
 	*tab->priv->status_message = '\0';
 	tab->priv->load_status = TAB_LOAD_NONE;
 	tab->priv->load_percent = 0;
@@ -216,7 +269,10 @@ galeon_tab_init (GaleonTab *tab)
 
 	/* set a pointer in the embed's widget back to the tab */
 	g_object_set_data (embed_widget, "GaleonTab", tab);
-
+	
+	g_signal_connect (embed_widget, "parent_set",
+			  G_CALLBACK (galeon_tab_parent_set_cb), 
+			  tab);
 	g_signal_connect (embed_widget, "destroy",
 			  GTK_SIGNAL_FUNC (galeon_tab_embed_destroy_cb), 
 			  tab);
@@ -265,6 +321,12 @@ galeon_tab_init (GaleonTab *tab)
 	g_signal_connect (embed, "ge_security_change",
 			  GTK_SIGNAL_FUNC (galeon_tab_security_change_cb), 
 			  tab);
+	g_signal_connect (embed, "ge_favicon",
+			  G_CALLBACK (galeon_tab_favicon_cb),
+			  tab);
+	g_signal_connect(embed, "ge_popupblocked",
+			 G_CALLBACK(galeon_tab_popup_blocked_cb),
+			 tab);
 }
 
 /* Destructor */
@@ -282,6 +344,9 @@ galeon_tab_finalize (GObject *object)
 
 	g_idle_remove_by_data (tab->priv->embed);
 	
+	g_free (tab->priv->link_message);
+	g_free (tab->priv->js_message);
+
         g_free (tab->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -307,15 +372,22 @@ static void
 galeon_tab_set_load_status (GaleonTab *tab,
 			    TabLoadStatus status)
 {
-	tab->priv->load_status &= TAB_LOAD_ACTIVATED;
-	tab->priv->load_status |= status;
-
-	if (tab->priv->load_status & TAB_LOAD_COMPLETED)
+	if (status == TAB_LOAD_COMPLETED)
 	{
 		Session *s;
 	    	s = galeon_shell_get_session (galeon_shell);
         	session_save (s, SESSION_CRASHED);
 	}
+
+	if (galeon_tab_get_is_active (tab) && 
+	    status == TAB_LOAD_COMPLETED)
+	{
+		tab->priv->load_status = TAB_LOAD_NONE;
+	}
+	else
+	{
+		tab->priv->load_status = status;
+	}
 }
 
 GaleonEmbed *
@@ -346,42 +418,24 @@ galeon_tab_get_window (GaleonTab *tab)
 static void
 galeon_tab_update_color (GaleonTab *tab)
 {
-	TabLoadStatus status = galeon_tab_get_load_status (tab);
-	GulNotebookPageLoadStatus page_status = 0;
-	GtkWidget *nb;
-
-	nb = galeon_window_get_notebook (tab->priv->window);
-	
-	if (status & TAB_LOAD_ACTIVATED)
-	{
-		page_status = GUL_NOTEBOOK_TAB_LOAD_ACTIVATED;
-	}
-	else if (status & TAB_LOAD_STARTED)
-        {
-       		page_status = GUL_NOTEBOOK_TAB_LOAD_STARTED;
-	}
-        else if (status & TAB_LOAD_COMPLETED)
-        {
-		page_status = GUL_NOTEBOOK_TAB_LOAD_COMPLETED;
-        }
-	else
-	{
-		g_assert_not_reached ();
-	}
-
-	gul_notebook_set_page_status (GUL_NOTEBOOK (nb),
-				      GTK_WIDGET (tab->priv->embed),
-				      page_status);
+	galeon_window_set_tab_status (tab->priv->window, tab, 
+			              tab->priv->load_status);
 }
 
 void
 galeon_tab_set_is_active (GaleonTab *tab, gboolean is_active)
 {
+	TabLoadStatus status;
+	
 	g_return_if_fail (IS_GALEON_TAB (G_OBJECT (tab)));
-
+	
 	tab->priv->is_active = is_active;
 
-	galeon_tab_set_load_status (tab, TAB_LOAD_ACTIVATED);
+       	status = galeon_tab_get_load_status (tab);
+	if (status == TAB_LOAD_COMPLETED)
+	{
+		galeon_tab_set_load_status (tab, TAB_LOAD_NONE);
+	}
 	galeon_tab_update_color (tab);
 }
 
@@ -425,13 +479,15 @@ galeon_tab_link_message_cb (GaleonEmbed *embed,
 			    GaleonTab *tab)
 {
 	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
-	if (!tab->priv->is_active) return;
 
-	g_strlcpy (tab->priv->status_message, 
-		   message, 255);
+	g_free (tab->priv->link_message);
+	tab->priv->link_message = g_strdup (message);
 	
-	galeon_window_update_control (tab->priv->window,
-				      StatusbarMessageControl);
+	if (tab->priv->is_active)
+	{
+		galeon_window_update_control (tab->priv->window,
+					      StatusbarMessageControl);
+	}
 }
 
 static void
@@ -439,39 +495,36 @@ galeon_tab_js_status_cb (GaleonEmbed *embed,
 			 const char *status,
 			 GaleonTab *tab)
 {
-	gboolean allow_statusbar_rewrite;
-	
 	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
 	
-	allow_statusbar_rewrite = eel_gconf_get_boolean 
-		(CONF_FILTERING_DEFAULT_STATUSBAR);
-	
-	if (!tab->priv->is_active || !allow_statusbar_rewrite)
-		return;
-
-	g_strlcpy (tab->priv->status_message, 
-		   status, 255);
-		
-	galeon_window_update_control (tab->priv->window,
-				      StatusbarMessageControl);
+	g_free (tab->priv->js_message);
+	tab->priv->js_message = g_strdup (status);
+			
+	if (tab->priv->is_active 
+	    && eel_gconf_get_boolean (CONF_FILTERING_DEFAULT_STATUSBAR))
+	{
+		galeon_window_update_control (tab->priv->window,
+					      StatusbarMessageControl);
+	}
 }
 
 static void
 galeon_tab_location_cb (GaleonEmbed *embed, GaleonTab *tab)
 {		
-	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+	CHECK_EMBED_TAB_WINDOW(embed, tab, tab->priv->window);
 
-	if (tab->priv->location) g_free (tab->priv->location);
-	galeon_embed_get_location (embed, TRUE, FALSE, 
-				   &tab->priv->location);
+	if (tab->priv->location) g_free(tab->priv->location);
+	galeon_embed_get_location(embed, TRUE, FALSE, 
+				  &tab->priv->location);
 
 	if (tab->priv->is_active)
 	{
-		galeon_window_update_control (tab->priv->window, LocationControl);
-		galeon_window_update_control (tab->priv->window, NavControl);
+		galeon_window_update_control(tab->priv->window, LocationControl);
+		galeon_window_update_control(tab->priv->window, NavControl);
+		galeon_window_update_control(tab->priv->window, PermissionControl);
 	}
 
-	galeon_window_site_visited (tab->priv->window, tab->priv->location, tab->priv->title);
+	galeon_window_site_visited(tab->priv->window, tab->priv->location, tab->priv->title);
 }
 
 static void
@@ -485,17 +538,6 @@ galeon_tab_zoom_changed_cb (GaleonEmbed *embed, gint zoom, GaleonTab *tab)
 	}
 }
 
-static void
-galeon_tab_set_title (GaleonTab *tab, const char *title)
-{
-	GtkWidget *nb;
-	
-	nb = galeon_window_get_notebook (tab->priv->window);
-	gul_notebook_set_page_title (GUL_NOTEBOOK (nb),
-				     GTK_WIDGET (tab->priv->embed),
-				     title);
-}
-
 static void
 galeon_tab_title_cb (GaleonEmbed *embed, GaleonTab *tab)
 {
@@ -512,13 +554,8 @@ galeon_tab_title_cb (GaleonEmbed *embed, GaleonTab *tab)
 	
 	galeon_window_site_visited (tab->priv->window, tab->priv->location, tab->priv->title);
 
-	galeon_tab_set_title (tab, tab->priv->title);
-	
-	if (tab->priv->is_active)
-	{
-		galeon_window_update_control (tab->priv->window, 
-					      TitleControl);
-	}
+	galeon_window_set_tab_title (tab->priv->window, 
+			             tab, tab->priv->title);
 }
 
 static int
@@ -597,7 +634,6 @@ galeon_tab_progress_cb (GaleonEmbed *embed, const char *uri,
 			GaleonTab *tab)
 {
 	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
-	if (!tab->priv->is_active) return;
 
 	build_progress_message (tab->priv->status_message,
 				uri,
@@ -605,8 +641,11 @@ galeon_tab_progress_cb (GaleonEmbed *embed, const char *uri,
 				maxprogress,
 				tab->priv->load_percent);
 	
-	galeon_window_update_control (tab->priv->window,
-				      StatusbarMessageControl);
+	if (tab->priv->is_active)
+	{
+		galeon_window_update_control (tab->priv->window,
+					      StatusbarMessageControl);
+	}
 }
 
 static void
@@ -699,8 +738,11 @@ galeon_tab_net_state_cb (GaleonEmbed *embed, const char *uri,
 {	
 	build_net_state_message (tab->priv->status_message, uri, state);
 
-	galeon_window_update_control (tab->priv->window,
-				      StatusbarMessageControl);
+	if (tab->priv->is_active)
+	{
+		galeon_window_update_control (tab->priv->window,
+					      StatusbarMessageControl);
+	}
 	
 	if (state & EMBED_STATE_IS_NETWORK)
 	{
@@ -713,9 +755,21 @@ galeon_tab_net_state_cb (GaleonEmbed *embed, const char *uri,
 			ensure_location (tab, uri);
 			
 			galeon_tab_set_load_status (tab, TAB_LOAD_STARTED);
+
+			galeon_window_update_control (tab->priv->window,
+						      NavControl);
 			galeon_window_update_control (tab->priv->window,
 						      SpinnerControl);
 			galeon_tab_update_color (tab);
+
+			if (tab->priv->blocked_popup_uri)
+			{
+				g_free(tab->priv->blocked_popup_uri);
+				tab->priv->blocked_popup_uri = NULL;
+			}
+			galeon_window_update_control(tab->priv->window,
+						     StatusbarPopupBlockControl);
+
 		}
 		else if (state & EMBED_STATE_STOP)
 		{			
@@ -794,6 +848,9 @@ galeon_tab_visibility_cb (GaleonEmbed *embed, gboolean visibility,
 	g_return_if_fail (window != NULL);
 	
 	galeon_window_update_control (window, WindowVisibilityControl);
+#if MOZILLA_SNAPSHOT <= 6
+	galeon_window_update_control(window, PermissionControl);
+#endif
 }
 
 static void
@@ -853,12 +910,19 @@ galeon_tab_size_to_cb (GaleonEmbed *embed, gint width, gint height,
 	}
 }
 
-static gint
-galeon_tab_dom_mouse_click_cb (GaleonEmbed *embed,
-			       GaleonEmbedEvent *event,
-			       GaleonTab *tab)
+static void
+galeon_tab_show_embed_popup (GaleonTab *tab, GaleonEmbedEvent *event)
 {
-	return FALSE;
+	GaleonEmbedPopup *popup;
+	GaleonWindow *window;
+	GaleonEmbed *embed;
+
+	window = galeon_tab_get_window (tab);
+	embed = galeon_tab_get_embed (tab);
+
+	popup = GALEON_EMBED_POPUP (galeon_window_get_popup_factory (window));
+	galeon_embed_popup_set_event (popup, event);
+	galeon_embed_popup_show (popup, embed);
 }
 
 static gint
@@ -866,62 +930,47 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 			       GaleonEmbedEvent *event,
 			       GaleonTab *tab)
 {
-	GaleonEmbedPopup *popup;
 	GaleonWindow *window;
-	BonoboWindow *bwin;
 	int button;
 	EmbedEventContext context;
         
-	galeon_embed_event_get_context (event, &context); 
-	
 	g_assert (IS_GALEON_EMBED_EVENT(event));
-	
-	galeon_embed_event_get_mouse_button (event, &button);
-	
+
 	window = galeon_tab_get_window (tab);
 	g_return_val_if_fail (window != NULL, FALSE);
 
-	bwin = BONOBO_WINDOW(window);
-	g_assert (bwin != NULL);
+	galeon_embed_event_get_mouse_button (event, &button);
+	galeon_embed_event_get_context (event, &context); 
 
 	if (button == 2)
 	{
-		gboolean gestures = eel_gconf_get_boolean (CONF_MOUSE_RIGHT_BUTTON_GESTURES);
-		
-		if (!gestures) /* bring up the menu immediately */
+		gint action = eel_gconf_get_integer (CONF_MOUSE_RIGHT_BUTTON_ACTION);
+		switch (action)
 		{
-			popup = GALEON_EMBED_POPUP (galeon_window_get_popup_factory (window));
-			galeon_embed_popup_set_event (popup, event);
-			galeon_embed_popup_show (popup, embed);
-		}
-		else /* start the gesture */
+		case 0: /* display context menu immediately */
+			galeon_tab_show_embed_popup (tab, event);
+			break;
+		case 1: /* perform gestures, show context menu on mouse up */
 		{ 
 			GulGestures *g = gul_gestures_new ();
 			guint x, y;
-			g_object_ref(event);
-			g_object_set_data_full(G_OBJECT(g), "embed_event", event, g_object_unref);
+			g_object_set_data_full(G_OBJECT(g), "embed_event", 
+					       g_object_ref (event), 
+					       g_object_unref);
 			
 			galeon_embed_event_get_mouse_coords (event, &x, &y);
-			gul_gestures_set_autocancel (g, TRUE);
-			gul_gestures_start (g, GTK_WIDGET (bwin), 3, x, y);
+			gul_gestures_start (g, GTK_WIDGET (window), button, x, y);
 			
 			g_signal_connect (g, "gesture-performed", 
 					  G_CALLBACK (galeon_tab_gesture_performed_cb), tab);
 
-			g_signal_connect (g, "cancelled", 
-					  G_CALLBACK (galeon_tab_gesture_cancelled_cb), tab);
-				
 			g_object_unref (g);
 		}
-	}
-	else if (button == 1
-		 && (context & EMBED_CONTEXT_LINK))
-	{
-		GValue *value;
-	
-		galeon_embed_event_get_property (event, "link", &value);
-		galeon_shell_new_tab (galeon_shell, window, tab, 
-				      g_value_get_string (value), 0);
+		break;
+		default:
+			g_warning ("Unexpected pref value for " CONF_MOUSE_RIGHT_BUTTON_ACTION "\n");
+			break;
+		}
 	}
 	else if (button == 1
 		&& !(context & EMBED_CONTEXT_LINK
@@ -934,11 +983,8 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		case 0: /* bookmarks context menu */
 			g_warning ("Bookmarks context menu is not implemented yet\n");
 			break;
-		case 1: /* paste url */
-			gtk_selection_convert (GTK_WIDGET (window), 
-					       GDK_SELECTION_PRIMARY,
-					       GDK_SELECTION_TYPE_STRING,
-					       GDK_CURRENT_TIME);
+		case 1: /* paste url: handled by click_cb */
+		case 6: /* paste url in current tab: handled by click_cb */
 			break;
 		case 2: /* go back */
 			galeon_embed_go_back (embed);
@@ -947,10 +993,8 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		{
 			GulGestures *g = gul_gestures_new ();
 			guint x, y;
-			g_object_set_data_full(G_OBJECT(g), "embed_event", event, g_object_unref);
-			g_object_ref(event);
 			galeon_embed_event_get_mouse_coords (event, &x, &y);
-			gul_gestures_start (g, GTK_WIDGET (bwin), 3, x, y);
+			gul_gestures_start (g, GTK_WIDGET (window), button, x, y);
 			
 			g_signal_connect (g, "gesture-performed", 
 					  G_CALLBACK (galeon_tab_gesture_performed_cb), tab);
@@ -964,7 +1008,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 			guint x, y;
 			galeon_embed_event_get_mouse_coords (event, &x, &y);
 			galeon_embed_autoscroller_set_embed (as, embed);
-			galeon_embed_autoscroller_start_scroll (as, GTK_WIDGET (bwin), x, y);
+			galeon_embed_autoscroller_start_scroll (as, GTK_WIDGET (window), x, y);
 			g_object_unref (as);
 		}
 		break;
@@ -974,7 +1018,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 			guint x, y;
 			galeon_embed_event_get_mouse_coords (event, &x, &y);
 			galeon_embed_manual_scroller_set_embed (ms, embed);
-			galeon_embed_manual_scroller_start_scroll (ms, GTK_WIDGET (bwin), x, y);
+			galeon_embed_manual_scroller_start_scroll (ms, GTK_WIDGET (window), x, y);
 			g_object_unref (ms);
 		}
 		break;
@@ -987,15 +1031,110 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 	return FALSE;
 }
 
+static gint
+galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
+			      GaleonEmbedEvent *event,
+			      GaleonTab *tab)
+{
+	GaleonWindow *window;
+	guint button, modifier;
+	EmbedEventContext context;
+        
+	g_assert(IS_GALEON_EMBED_EVENT(event));
+
+	window = galeon_tab_get_window(tab);
+	g_return_val_if_fail(window != NULL, FALSE);
+
+	galeon_embed_event_get_mouse_button(event, &button);
+	galeon_embed_event_get_context(event, &context);
+	galeon_embed_event_get_modifier(event, &modifier);
+
+	if (button == 0 && (context & EMBED_CONTEXT_LINK)
+	    && (modifier & GDK_SHIFT_MASK))
+	{
+		GaleonEmbedPersist *persist;
+		const GValue *value;
+		gboolean ask_dir;
+
+		galeon_embed_event_get_property(event, "link", &value);
+		if (eel_gconf_get_boolean(CONF_DOWNLOADING_EXTERNAL_DOWNLOADER) ==
+		    DOWNLOADER_EXTERNAL)
+		{
+			gul_download_external_save_url(g_value_get_string(value));
+		}
+		else
+		{
+			persist = galeon_embed_persist_new(embed);
+			galeon_embed_persist_set_source(persist,
+							g_value_get_string(value));
+			galeon_embed_persist_set_flags(persist,
+						       EMBED_PERSIST_SHOW_PROGRESS);
+
+			ask_dir = eel_gconf_get_boolean(CONF_DOWNLOADING_ASK_DIR);
+
+			galeon_embed_utils_save(GTK_WIDGET(window),
+						CONF_STATE_SAVE_DIR,
+						ask_dir, FALSE, persist);
+
+			g_object_unref(G_OBJECT(persist));
+		}
+	}
+
+	else if (button == 1 && (context & EMBED_CONTEXT_LINK))
+	{
+		GaleonNewTabFlags flags;
+		const GValue *value;
+
+		flags = galeon_shell_modifier_flags(modifier);
+		galeon_embed_event_get_property(event, "link", &value);
+		galeon_shell_new_tab(galeon_shell, window, tab, 
+				     g_value_get_string (value), 
+				     flags);
+	}
+	else if (button == 1 &&
+		 (eel_gconf_get_integer(CONF_MOUSE_MIDDLE_BUTTON_ACTION) == 1 ||
+		  eel_gconf_get_integer(CONF_MOUSE_MIDDLE_BUTTON_ACTION) == 6) &&
+		 !(context & EMBED_CONTEXT_LINK ||
+		   context & EMBED_CONTEXT_EMAIL_LINK ||
+		   context & EMBED_CONTEXT_INPUT))
+	{
+		gtk_selection_convert(GTK_WIDGET(window), 
+				      GDK_SELECTION_PRIMARY,
+				      GDK_SELECTION_TYPE_STRING,
+				      GDK_CURRENT_TIME);
+	}
+	return FALSE;
+}
+
 static void
 galeon_tab_security_change_cb (GaleonEmbed *embed, EmbedSecurityLevel level,
 			       GaleonTab *tab)
 {
 	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
-	if (!tab->priv->is_active) return;
 	
-	galeon_window_update_control (tab->priv->window,
-				      StatusbarSecurityControl);
+	if (tab->priv->is_active)
+	{
+		galeon_window_update_control (tab->priv->window,
+					      StatusbarSecurityControl);
+	}
+}
+
+static void
+galeon_tab_popup_blocked_cb(GaleonEmbed *embed, const char *uri,  
+                            GaleonTab *tab)
+{
+        CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+
+	if (tab->priv->blocked_popup_uri)
+		g_free(tab->priv->blocked_popup_uri);
+
+	tab->priv->blocked_popup_uri = uri ? g_strdup(uri) : NULL;
+
+        if (tab->priv->is_active)
+        {
+                galeon_window_update_control(tab->priv->window,
+                                             StatusbarPopupBlockControl);
+        }
 }
 
 TabLoadStatus 
@@ -1013,14 +1152,25 @@ galeon_tab_get_load_percent (GaleonTab *tab)
 const char *
 galeon_tab_get_status_message (GaleonTab *tab)
 {
-	if (tab->priv->status_message)
+	/* The statusbar message is the first one of the following:
+	 * 1) the URL if hovering the mouse over a link
+	 * 2) the javascript message if overwriting the statusbar is allowed
+	 * 3) normal status message (Loading..., Done, etc.)
+	 *
+	 * Note: CONF_FILTERING_DEFAULT_STATUSBAR is basically obsolete since
+	 * we don't let people 'fake' URLs by rewriting the statusbar.
+	 */
+
+	if (*tab->priv->link_message)
 	{
-		return tab->priv->status_message;
+		return tab->priv->link_message;
 	}
-	else
+	else if (*tab->priv->js_message
+		 && eel_gconf_get_boolean (CONF_FILTERING_DEFAULT_STATUSBAR))
 	{
-		return " ";
+		return tab->priv->js_message;
 	}
+	return tab->priv->status_message;
 }
 
 const char *
@@ -1043,6 +1193,22 @@ galeon_tab_get_location (GaleonTab *tab)
 	return tab->priv->location;
 }
 
+const char *
+galeon_tab_get_blocked_popup_uri(GaleonTab *tab)
+{
+	return tab->priv->blocked_popup_uri;
+}
+
+void
+galeon_tab_clear_blocked_popup_uri(GaleonTab *tab)
+{
+	if (tab->priv->blocked_popup_uri)
+	{
+		g_free(tab->priv->blocked_popup_uri);
+		tab->priv->blocked_popup_uri = NULL;
+	}
+}
+
 void 
 galeon_tab_set_location (GaleonTab *tab,
 			 char *location)
@@ -1051,18 +1217,6 @@ galeon_tab_set_location (GaleonTab *tab,
 	tab->priv->location = location;
 }
 
-void 
-galeon_tab_update_control (GaleonTab *tab,
-			   TabControlID id)
-{
-	switch (id)
-	{
-		case TAB_CONTROL_TITLE:
-			galeon_tab_set_title (tab, tab->priv->title);
-			break;
-	}
-}
-
 static void
 galeon_tab_update_gestures (void)
 {
@@ -1200,7 +1354,9 @@ galeon_tab_update_gestures (void)
 static void
 galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTab *tab)
 {
-	const gchar *action;
+	const gchar  *action;
+	GaleonWindow *window;
+	GaleonEmbed  *embed;
 	
 	DEBUG_MSG (("gesture: %s\n", sequence));
 	
@@ -1217,27 +1373,19 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	
 	DEBUG_MSG (("action: %s\n", action));
+	window = galeon_tab_get_window (tab);
+	embed = galeon_tab_get_embed (tab);
 	
 	if (!strcmp (action, "menu"))
 	{
 		GaleonEmbedEvent *event = g_object_get_data(G_OBJECT(g), "embed_event");
-		GaleonWindow *window;
-		GaleonEmbed *embed;
-		GaleonEmbedPopup *popup;
-		
-		window = galeon_tab_get_window (tab);
-		embed = galeon_tab_get_embed (tab);
-      
-		popup = GALEON_EMBED_POPUP (galeon_window_get_popup_factory (window));
-		galeon_embed_popup_set_embed(popup, embed);
-		galeon_embed_popup_set_event (popup, event);
-		galeon_embed_popup_show (popup, embed);
-
+		if (event != NULL)
+		{
+			galeon_tab_show_embed_popup (tab, event);
+		}
 	}
 	else if (!strcmp (action, "new_tab"))
 	{
-		GaleonWindow *window = galeon_tab_get_window (tab);
-		
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL, 
 				      GALEON_NEW_TAB_HOMEPAGE |
 				      GALEON_NEW_TAB_IN_EXISTING_WINDOW |
@@ -1245,8 +1393,6 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "new_window"))
 	{
-		GaleonWindow *window = galeon_tab_get_window (tab);
-		
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL,
 				      GALEON_NEW_TAB_HOMEPAGE |
 				      GALEON_NEW_TAB_IN_NEW_WINDOW |
@@ -1254,15 +1400,24 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "reload"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_embed_reload (embed, EMBED_RELOAD_NORMAL);
 	}
 	else if (!strcmp (action, "reload_bypass"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_embed_reload (embed, 
+				     EMBED_RELOAD_BYPASSCACHE |
+				     EMBED_RELOAD_BYPASSPROXY);
 	}
 	else if (!strcmp (action, "homepage"))
 	{
-		NOT_IMPLEMENTED;
+		char *homepage;
+
+		homepage = eel_gconf_get_string (CONF_GENERAL_HOMEPAGE);
+		g_return_if_fail (homepage != NULL);
+
+		galeon_embed_load_url (embed, homepage);
+
+		g_free (homepage);
 	}
 	else if (!strcmp (action, "clone_window"))
 	{
@@ -1270,8 +1425,6 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "clone_tab"))
 	{
-		GaleonWindow *window = galeon_tab_get_window (tab);
-
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL,
 				      GALEON_NEW_TAB_IN_EXISTING_WINDOW |
 				      GALEON_NEW_TAB_JUMP |
@@ -1279,36 +1432,34 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "up"))
 	{
-		galeon_embed_go_up (galeon_tab_get_embed (tab));
+		galeon_embed_go_up (embed);
 	}
 	else if (!strcmp (action, "close"))
 	{
-		galeon_window_remove_tab (galeon_tab_get_window (tab), tab);
+		galeon_window_remove_tab (window, tab);
 	}
 	else if (!strcmp (action, "back"))
 	{
-		galeon_embed_go_back (galeon_tab_get_embed (tab));
+		galeon_embed_go_back (embed);
 	}
 	else if (!strcmp (action, "forward"))
 	{
-		galeon_embed_go_forward (galeon_tab_get_embed (tab));
+		galeon_embed_go_forward (embed);
 	}
 	else if (!strcmp (action, "fullscreen"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_window_toggle_fullscreen (window);
 	}
-	else if (!strcmp (action, "new_tab"))
+	else if (!strcmp (action, "next_tab"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_window_next_tab (window);
 	}
 	else if (!strcmp (action, "prev_tab"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_window_prev_tab (window);
 	}
 	else if (!strcmp (action, "view_source"))
 	{
-		GaleonWindow *window = galeon_tab_get_window (tab);
-
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL,
 				      GALEON_NEW_TAB_IN_EXISTING_WINDOW |
 				      GALEON_NEW_TAB_JUMP |
@@ -1316,7 +1467,7 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "stop"))
 	{
-		galeon_embed_stop_load (galeon_tab_get_embed (tab));
+		galeon_embed_stop_load (embed);
 	}
 	else if (!strcmp (action, "prev_link"))
 	{
@@ -1335,20 +1486,3 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 		/* unrecognized */
 	}
 }
-
-static void
-galeon_tab_gesture_cancelled_cb (GulGestures *ges, GaleonTab *tab)
-{
-	GaleonEmbedEvent *event = g_object_get_data (G_OBJECT (ges), "embed_event");
-	GaleonWindow *window;
-	GaleonEmbed *embed;
-	GaleonEmbedPopup *popup;
-	
-	window = galeon_tab_get_window (tab);
-	embed = galeon_tab_get_embed (tab);
-	
-	popup = GALEON_EMBED_POPUP (galeon_window_get_popup_factory (window));
-	galeon_embed_popup_set_embed(popup, embed);
-	galeon_embed_popup_set_event (popup, event);
-	galeon_embed_popup_show (popup, embed);
-}	
