@@ -95,6 +95,9 @@ const char   *galeon_tab_get_location           (GaleonTab *tab);
 void	      galeon_tab_set_location           (GaleonTab *tab,
 						 char *location);
 
+const char   *galeon_tab_get_blocked_popup_uri  (GaleonTab *tab);
+void	     galeon_tab_clear_blocked_popup_uri (GaleonTab *tab);
+
 void	      galeon_tab_get_size		(GaleonTab *tab,
 						 int *width,
 						 int *height);
