@@ -72,13 +72,15 @@
 #define VIEW_TOOLBAR_PATH "/commands/View Toolbar"
 #define VIEW_BOOKMARKS_TOOLBAR_PATH "/commands/View Bookmarks Toolbars"
 #define VIEW_FULLSCREEN_PATH "/commands/View Fullscreen"
-#define ALLOW_POPUPS_PATH "/commands/Allow Popups"
+#define ALLOW_POPUPS_TOGGLE_PATH "/commands/Allow Popups"
 #define ALLOW_JAVA_PATH "/commands/Allow Java"
 #define ALLOW_JAVASCRIPT_PATH "/commands/Allow JavaScript"
+#define ALLOW_POPUPS_CMD_PATH "/commands/AllowPopupSite"
 #define ALLOW_COOKIES_CMD_PATH "/commands/AllowCookieSite"
 #define BLOCK_COOKIES_CMD_PATH "/commands/BlockCookieSite"
 #define ALLOW_IMAGES_CMD_PATH "/commands/AllowImageSite"
 #define BLOCK_IMAGES_CMD_PATH "/commands/BlockImageSite"
+#define ALLOW_POPUPS_MENU_PATH "/menu/Web/Popups/AllowPopupSite"
 #define ALLOW_COOKIES_MENU_PATH "/menu/Web/Cookies/AllowCookieSite"
 #define BLOCK_COOKIES_MENU_PATH "/menu/Web/Cookies/BlockCookieSite"
 #define ALLOW_IMAGES_MENU_PATH "/menu/View/Images/AllowImageSite"
@@ -139,6 +141,7 @@ static BonoboUIVerb galeon_verbs [] = {
 	BONOBO_UI_VERB ("Zoom Out", (BonoboUIVerbFn)window_cmd_view_zoom_out),
 	BONOBO_UI_VERB ("Zoom Normal", (BonoboUIVerbFn)window_cmd_view_zoom_normal),
 	BONOBO_UI_VERB ("ViewPageSource", (BonoboUIVerbFn)window_cmd_view_page_source),
+	BONOBO_UI_VERB ("AllowPopupSite", (BonoboUIVerbFn)window_cmd_set_permission),
 	BONOBO_UI_VERB ("BlockImageSite", (BonoboUIVerbFn)window_cmd_set_permission),
 	BONOBO_UI_VERB ("AllowImageSite", (BonoboUIVerbFn)window_cmd_set_permission),
 	BONOBO_UI_VERB ("BlockCookieSite", (BonoboUIVerbFn)window_cmd_set_permission),
@@ -154,6 +157,7 @@ static BonoboUIVerb galeon_verbs [] = {
 	BONOBO_UI_VERB ("TabsMoveLeft", (BonoboUIVerbFn)window_cmd_tabs_move_left),
 	BONOBO_UI_VERB ("TabsMoveRight", (BonoboUIVerbFn)window_cmd_tabs_move_right),
 	BONOBO_UI_VERB ("TabsDetach", (BonoboUIVerbFn)window_cmd_tabs_detach),
+	BONOBO_UI_VERB ("TabsClone", (BonoboUIVerbFn)window_cmd_tabs_clone),
 	BONOBO_UI_VERB ("HelpContents", (BonoboUIVerbFn)window_cmd_help_manual),
 	BONOBO_UI_VERB ("About", (BonoboUIVerbFn)window_cmd_help_about),
 
@@ -285,6 +289,8 @@ galeon_window_key_press_event_cb (GtkWidget *widget,
 {
         int page;
 
+	g_print("Window keypress\n");
+
 	if ((event->state & GDK_Shift_L) || (event->state & GDK_Shift_R))
                 return FALSE;
 
@@ -707,12 +713,34 @@ allow_setting_gconf_changed_cb(GConfClient *client,
 			       GConfEntry *entry,
 			       BonoboUIComponent *ui_component)
 {
-	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_PATH,
-				    eel_gconf_get_boolean (CONF_FILTERING_ALLOW_POPUPS));
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_TOGGLE_PATH,
+				    eel_gconf_get_boolean(CONF_FILTERING_ALLOW_POPUPS));
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVASCRIPT_PATH,
-				    eel_gconf_get_boolean (CONF_FILTERING_JAVASCRIPT_ENABLED));
+				    eel_gconf_get_boolean(CONF_FILTERING_JAVASCRIPT_ENABLED));
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVA_PATH,
-				    eel_gconf_get_boolean (CONF_FILTERING_JAVA_ENABLED));
+				    eel_gconf_get_boolean(CONF_FILTERING_JAVA_ENABLED));
+}
+
+//Prototype needed as implemented later
+static void update_permission_control(GaleonWindow *window);
+
+static void
+allow_popups_bonoboui_changed_cb(BonoboUIComponent *component,
+				 const char *path,
+				 Bonobo_UIComponent_EventType type,
+				 const char *state,
+				 GaleonWindow *window)
+{
+	update_permission_control(window);
+}
+
+static void
+allow_popups_gconf_changed_cb(GConfClient *client,
+			      guint cnxn_id,
+			      GConfEntry *entry,
+			      GaleonWindow *window)
+{
+	update_permission_control(window);
 }
 
 static void
@@ -762,13 +790,16 @@ setup_web_menu (GaleonWindow *window)
 				   (GConfClientNotifyFunc)accept_cookies_gconf_changed_cb, 
 				   ui_component);
 
-	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_PATH,
+	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_TOGGLE_PATH,
 				    eel_gconf_get_boolean (CONF_FILTERING_ALLOW_POPUPS));
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVASCRIPT_PATH,
 				    eel_gconf_get_boolean (CONF_FILTERING_JAVASCRIPT_ENABLED));
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_JAVA_PATH,
 				    eel_gconf_get_boolean (CONF_FILTERING_JAVA_ENABLED));
 
+	bonobo_ui_component_add_listener(ui_component, ID_ALLOW_POPUPS,
+					 (BonoboUIListenerFn)allow_popups_bonoboui_changed_cb,
+					 window);
 	bonobo_ui_component_add_listener(ui_component, ID_ALLOW_POPUPS,
 					 (BonoboUIListenerFn)allow_setting_bonoboui_changed_cb,
 					 CONF_FILTERING_ALLOW_POPUPS);
@@ -779,6 +810,9 @@ setup_web_menu (GaleonWindow *window)
 					 (BonoboUIListenerFn)allow_setting_bonoboui_changed_cb,
 					 CONF_FILTERING_JAVA_ENABLED);
 
+	eel_gconf_notification_add(CONF_FILTERING_ALLOW_POPUPS,
+				   (GConfClientNotifyFunc)allow_popups_gconf_changed_cb,
+				   window);
 	eel_gconf_notification_add(CONF_FILTERING_ALLOW_POPUPS,
 				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
 				   ui_component);
@@ -1572,6 +1606,21 @@ update_security (GaleonWindow *window)
 	g_free (tooltip);
 }
 
+static void
+update_popup_block(GaleonWindow *window)
+{
+	GaleonTab *tab;
+	const char *uri = NULL;
+
+	tab = galeon_window_get_active_tab (window);
+	g_return_if_fail (tab != NULL);
+
+	uri = galeon_tab_get_blocked_popup_uri(tab);
+
+	statusbar_set_popup_block(STATUSBAR(window->priv->statusbar),
+				  uri ? TRUE : FALSE, uri);
+}
+
 static void
 update_nav_control (GaleonWindow *window)
 {
@@ -1648,7 +1697,8 @@ update_location_control (GaleonWindow *window)
 	GaleonTab *tab;
 	const char *location, *host = NULL;
 	char *allow_images_str, *block_images_str,
-	     *allow_cookies_str, *block_cookies_str;
+	     *allow_cookies_str, *block_cookies_str,
+	     *allow_popups_str;
 	GnomeVFSURI *uri = NULL;
 
 	tab = galeon_window_get_active_tab (window);
@@ -1681,6 +1731,7 @@ update_location_control (GaleonWindow *window)
 	block_cookies_str = g_strdup_printf(_("_Block Cookies From %s"), host);
 	allow_images_str = g_strdup_printf(_("_Allow Images From %s"), host);
 	block_images_str = g_strdup_printf(_("_Block Images From %s"), host);
+	allow_popups_str = g_strdup_printf(_("_Allow Popups From %s"), host);
 
 	if (uri)
 		gnome_vfs_uri_unref(uri);
@@ -1693,11 +1744,14 @@ update_location_control (GaleonWindow *window)
 			     ALLOW_IMAGES_MENU_PATH, allow_images_str);
 	gul_bonobo_set_label(BONOBO_UI_COMPONENT(window->ui_component),
 			     BLOCK_IMAGES_MENU_PATH, block_images_str);
+	gul_bonobo_set_label(BONOBO_UI_COMPONENT(window->ui_component),
+			     ALLOW_POPUPS_MENU_PATH, allow_popups_str);
 
 	g_free(allow_cookies_str);
 	g_free(block_cookies_str);
 	g_free(allow_images_str);
 	g_free(block_images_str);
+	g_free(allow_popups_str);
 }
 
 static void
@@ -1786,7 +1840,7 @@ update_permission_control (GaleonWindow *window)
 {
 	GaleonEmbedShell *embed_shell;
 	gresult result;
-	PermissionActionType image_action, cookie_action;
+	PermissionActionType image_action, cookie_action, popup_action;
 	GaleonTab *tab = galeon_window_get_active_tab(window);
 	if (tab == NULL) return;
 
@@ -1821,6 +1875,33 @@ update_permission_control (GaleonWindow *window)
 					 (cookie_action != DENY_ACTION));
 	}
 
+	//If global allow popups, then the site specific menu item
+	//is always insensitive
+	if (eel_gconf_get_boolean(CONF_FILTERING_ALLOW_POPUPS))
+	{
+		gul_bonobo_set_sensitive(BONOBO_UI_COMPONENT(window->ui_component),
+					 ALLOW_POPUPS_CMD_PATH,
+					 FALSE);
+		galeon_tab_clear_blocked_popup_uri(tab);
+		update_popup_block(window);
+		return;
+	}
+	
+	result = galeon_embed_shell_test_permission(embed_shell,
+						    galeon_tab_get_location(tab),
+						    POPUPS_PERMISSION,
+						    &popup_action);
+	if (result == G_OK)
+	{
+		gul_bonobo_set_sensitive(BONOBO_UI_COMPONENT(window->ui_component),
+					 ALLOW_POPUPS_CMD_PATH,
+					 (popup_action != ALLOW_ACTION));
+		if (popup_action == ALLOW_ACTION)
+		{
+			galeon_tab_clear_blocked_popup_uri(tab);
+			update_popup_block(window);
+		}
+	}
 }
 
 void  
@@ -1840,6 +1921,9 @@ galeon_window_update_control (GaleonWindow *window,
 	case StatusbarSecurityControl:
 		update_security (window);
 		break;
+	case StatusbarPopupBlockControl:
+		update_popup_block(window);
+		break;
 	case FindControl:
 		update_find_control (window);
 		break;
@@ -1888,6 +1972,7 @@ galeon_window_update_all_controls (GaleonWindow *window)
 		update_status_message (window);
 		update_progress (window);
 		update_security (window);
+		update_popup_block(window);
 		update_find_control (window);
 		update_spinner_control (window);
 		update_permission_control(window);
