@@ -358,7 +358,7 @@ toolbar_setup_spinner (Toolbar *t, GtkWidget *w)
 static void
 toolbar_setup_style (Toolbar *t)
 {
-	static const char *styles[] = { "both_horiz", "both", "text", "icon" };
+	static const char *styles[] = { "both_horiz", "both", "text", "icons" };
 	const char *style;
 	GConfValue *value;
 
@@ -374,9 +374,8 @@ toolbar_setup_style (Toolbar *t)
 			style = styles[i];
 		}
 		gconf_value_free (value);
+		gul_bonobo_set_look (t->priv->ui_component, "/Toolbar", style);
 	}
-
-	gul_bonobo_set_look (t->priv->ui_component, "/Toolbar", style);
 }
 
 static void
