@@ -629,7 +629,7 @@ window_cmd_set_permission(BonoboUIComponent *uic,
 					 TRUE);
 #endif
 	}
-	else //if (!strcmp(verbname, "BlockImageSite"))
+	else if (!strcmp(verbname, "BlockImageSite"))
 	{
 		type = IMAGES_PERMISSION;
 		value = FALSE;
@@ -640,6 +640,15 @@ window_cmd_set_permission(BonoboUIComponent *uic,
 					 FALSE);
 #endif
 	}
+	else //if (!strcmp(verbname, "AllowPopupSite"))
+	{
+		type = POPUPS_PERMISSION;
+		value = TRUE;
+#if MOZILLA_SNAPSHOT <=6
+		gul_bonobo_set_sensitive(uic, "/commands/AllowPopupSite",
+					 FALSE);
+#endif
+	}
 
 	galeon_embed_shell_set_permission(embed_shell, url, type, value);
 }
@@ -1002,6 +1011,22 @@ void window_cmd_tabs_move_right (BonoboUIComponent *uic,
 	}
 }
 
+void
+window_cmd_tabs_clone(BonoboUIComponent *uic,
+		      GaleonWindow *window,
+		      const char *verbname)
+{
+        GaleonTab *tab;
+
+        tab = galeon_window_get_active_tab(window);
+        g_return_if_fail(tab != NULL);
+
+	galeon_shell_new_tab(galeon_shell, window, tab, NULL,
+			     GALEON_NEW_TAB_IN_EXISTING_WINDOW |
+			     GALEON_NEW_TAB_JUMP |
+			     GALEON_NEW_TAB_IS_A_COPY);
+}
+
 void 
 window_cmd_tabs_detach  (BonoboUIComponent *uic, 
 			 GaleonWindow *window, 
